/******************************************************************************
 * @file:    HTTP_demo.c
 * @purpose: CMSIS Cortex-M3 Core Peripheral Access Layer Source File
 *           HTTP demo using CMSIS EthDev
 * @version: V1.01
 * @date:    22. May 2009
 *----------------------------------------------------------------------------
 *
 * Copyright (C) 2008 ARM Limited. All rights reserved.
 *
 * ARM Limited (ARM) is supplying this software for use with Cortex-M3 
 * processor based microcontrollers.  This file can be freely distributed 
 * within development tools that are supporting such ARM based processors. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ******************************************************************************/

#include "lm3s_cmsis.h"
#include <stdio.h>
#include <RTL.h>
#include <Net_Config.h>
#ifdef LM3S6965_RevC
  #include "rit128x96x4.h"
#else
  #include "osram128x64x4.h"
#endif
#include "EthDev.h"

/*---local variables----------------------------------------------------------*/
uint32_t LEDval;
BOOL     LEDrun;
BOOL     LCDupdate;
BOOL     tick;
uint32_t  dhcp_tout;
uint8_t   lcd_text[6][16+1] = {"    RL-ARM",         /* Buffer for LCD text   */
                               "  HTTP example",
                               " ",
                               "Waiting for Link",
                               " ",
                               " "
                              };

/*---Interface to ETh CMSIS --------------------------------------------------*/
extern EthDev_IOB eth0;
EthDev_IOB *pEth;
EthDev_STATUS EthState;


/*---Interface to TCP Stack---------------------------------------------------*/
extern LOCALM localm[];                             /* Local Machine Settings */
#define MY_IP localm[NETIF_ETH].IpAdr
#define DHCP_TOUT   50                              /* DHCP timeout 5 seconds */


volatile uint32_t msTicks;                          /* counts 100ms timeTicks */
/*----------------------------------------------------------------------------
  SysTick_Handler
 *----------------------------------------------------------------------------*/
void SysTick_Handler(void) {
  msTicks++;                        /* increment counter necessary in Delay() */
}

/*----------------------------------------------------------------------------
  delays number of tick Systicks (happens every 100 ms)
 *----------------------------------------------------------------------------*/
__inline static void Delay (uint32_t dlyTicks) {
  uint32_t curTicks;

  curTicks = msTicks;
  while ((msTicks - curTicks) < dlyTicks);
}

/*----------------------------------------------------------------------------
  configer LED pins
 *----------------------------------------------------------------------------*/
__inline static LED_init(void) {
  int i;

  SYSCTL->RCGC2 |= (1<<5);                      /* enable clock for GPIOF */
  for (i = 0; i < 10; i++) __NOP;               /* wait a while */

  GPIOF->DR2R   |= (1<<0);                      /* PIN0 strength 2MA */
  GPIOF->DEN    |= (1<<0);                      /* PIN0 is push-pull */
  GPIOF->DIR    |= (1<<0);                      /* PIN0 is output */
}


/*--------------------------- upd_display -----------------------------------*/

static void upd_display () {
   /* Update LCD Module display text. */

#ifdef LM3S6965_RevC
   RIT128x96x4Clear ();
   RIT128x96x4StringDraw ((const char *)&lcd_text[0], 20, 10, 11);
   RIT128x96x4StringDraw ((const char *)&lcd_text[1], 20, 25, 11);
   RIT128x96x4StringDraw ((const char *)&lcd_text[2], 20, 40, 11);
   RIT128x96x4StringDraw ((const char *)&lcd_text[3], 20, 55, 11);
   RIT128x96x4StringDraw ((const char *)&lcd_text[4], 20, 70, 11);
   RIT128x96x4StringDraw ((const char *)&lcd_text[5], 20, 85, 11);
#else
   OSRAM128x64x4Clear ();
   OSRAM128x64x4StringDraw ((const char *)&lcd_text[0], 20, 0, 11);
   OSRAM128x64x4StringDraw ((const char *)&lcd_text[1], 20, 10, 11);
   OSRAM128x64x4StringDraw ((const char *)&lcd_text[2], 20, 20, 11);
   OSRAM128x64x4StringDraw ((const char *)&lcd_text[3], 20, 30, 11);
   OSRAM128x64x4StringDraw ((const char *)&lcd_text[4], 20, 40, 11);
   OSRAM128x64x4StringDraw ((const char *)&lcd_text[5], 20, 50, 11);
#endif
   LCDupdate =__FALSE;
}


/*--------------------------- init_display ----------------------------------*/

static void init_display () {
   /* OLED Module init */

#ifdef LM3S6965_RevC
   RIT128x96x4Init(1000000);
#else
   OSRAM128x64x4Init(1000000);
#endif
   upd_display ();
}


/*----------------------------------------------------------------------------
  LED_out
 *----------------------------------------------------------------------------*/
void LED_out (uint32_t led) {
     GPIOF->DATA_Bits[0x01] = led & 1;
}


/*----------------------------------------------------------------------------
  Timer polling
 *----------------------------------------------------------------------------*/
static void timer_poll () {
  static uint32_t msTicksLast;

  if (msTicksLast != msTicks)  {                /* Timer tick every 100 ms */
      msTicksLast = msTicks;
      timer_tick ();
      tick = __TRUE;
   }
}


/*----------------------------------------------------------------------------
  Blink the LEDs on an eval board
 *----------------------------------------------------------------------------*/
static void LED_run () {
   /* Blink the LEDs on an eval board */
   static U32 LEDstat = 1;

   if (tick == __TRUE) {
      /* Every 100 ms */
      tick = __FALSE;
      if (LEDrun == __TRUE) {
         LEDstat = ~LEDstat & 0x01;
         LED_out (LEDstat);
      }
      if (LCDupdate == __TRUE) {
         upd_display ();
      }
   }
}


/*----------------------------------------------------------------------------
  Check the Link status
 *----------------------------------------------------------------------------*/
void link_check (void) {
   EthDev_STATUS newState;

   if (tick == __FALSE) {
      return;
   }
   /* Every 100 ms */
   newState = pEth->LinkChk ();
   if (newState.Link == EthState.Link) {
      /* No change. */
      return;
   }
   EthState = newState;

   if (EthState.Link == EthDev_LINK_DOWN) {
      MY_IP[0] = MY_IP[1] =  MY_IP[2] = MY_IP[3] = 0;          /* reset IP address */
      sprintf((char *)lcd_text[2]," ");
      sprintf((char *)lcd_text[3],"Waiting for Link");
      sprintf((char *)lcd_text[4]," ");
      sprintf((char *)lcd_text[5]," ");
      LCDupdate = __TRUE;
      pEth->UnInit ();
      init_TcpNet ();
   }
   else {
      sprintf((char *)lcd_text[2]," ");
      sprintf((char *)lcd_text[3],"Waiting for DHCP");
      sprintf((char *)lcd_text[4]," ");
      sprintf((char *)lcd_text[5]," ");
      LCDupdate = __TRUE;
      dhcp_tout = DHCP_TOUT;
   }
}


/*----------------------------------------------------------------------------
  Monitor DHCP IP address assignment
 *----------------------------------------------------------------------------*/
static void dhcp_check () {
   /* Monitor DHCP IP address assignment. */

   if (tick == __FALSE || dhcp_tout == 0) {
      return;
   }

   if (mem_test (&MY_IP, 0, IP_ADRLEN) == __FALSE && !(dhcp_tout & 0x80000000)) {
      /* Success, DHCP has already got the IP address. */
      dhcp_tout = 0;
      sprintf((char *)lcd_text[2],"IP address:");
      sprintf((char *)lcd_text[3],"%d.%d.%d.%d", MY_IP[0], MY_IP[1],
                                               MY_IP[2], MY_IP[3]);
      sprintf((char *)lcd_text[4],((EthState.Speed  == EthDev_SPEED_100M) ?
                                    "speed : 100M" : "speed: 10M"));
      sprintf((char *)lcd_text[5],((EthState.Duplex == EthDev_DUPLEX_FULL)?
                                    "duplex: FULL" : "duplex: HALF"));
      LCDupdate = __TRUE;
      return;
   }
   if (--dhcp_tout == 0) {
      /* A timeout, disable DHCP and use static IP address. */
      dhcp_disable ();
      sprintf((char *)lcd_text[3]," DHCP failed    " );
      LCDupdate = __TRUE;
      dhcp_tout = 30 | 0x80000000;              /* set timer to 3 sec to display message */
      return;
   }
   if (dhcp_tout == 0x80000000) {
      dhcp_tout = 0;
      sprintf((char *)lcd_text[2],"IP address:");
      sprintf((char *)lcd_text[3],"%d.%d.%d.%d", MY_IP[0], MY_IP[1],
                                               MY_IP[2], MY_IP[3]);
      sprintf((char *)lcd_text[4],((EthState.Speed  == EthDev_SPEED_100M) ?
                                    "speed : 100M" : "speed: 10M"));
      sprintf((char *)lcd_text[5],((EthState.Duplex == EthDev_DUPLEX_FULL)?
                                    "duplex: FULL" : "duplex: HALF"));
      LCDupdate = __TRUE;
   }
}


/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main (void) {

  SystemInit();									 /* setup clocks */
  if (SysTick_Config(SystemFrequency / 10)) {    /* Setup SysTick Timer for 100 msec interrupts  */
    while (1);                                   /* Capture error */
  }

  init_display ();
  LED_init();                             

  pEth = &eth0;                                  /* Select 'eth0' as active communication interface. */
  EthState.Link = EthDev_LINK_DOWN;
  init_TcpNet ();

  dhcp_tout = DHCP_TOUT;
  LEDrun    = __TRUE;

  while (1) {
    LED_run ();
    switch (EthState.Link) {
      case 0:                                    /* link down */
        timer_poll ();
        link_check ();
        break;
  
      case 1:                                    /* link up */
        timer_poll ();
        link_check ();
        main_TcpNet ();
        dhcp_check ();
        break;
     }
  }

}
