#! /bin/sh

# This script applies changes to the Sourcery G++ Lite 2008q3-39 source code
# in order to compile on Mac OS X 10.5, then builds and installs the various
# components.  Although derived from the same source, please note that this
# port is an independent project and should not be confused for an official
# CodeSourcery product.  If you encounter difficulties, please contact the
# script author at the address below - CodeSourcery is not liable and should
# not be burdened with support.  The file 'COPYING' and its variants,
# appearing in several locations throughout the source code, detail license
# restrictions, requirements, and legal disclaimers.

# 10/15/2008 - Phillip Burgess - pburgess@dslextreme.com

# Confirm that script is being run as root
if test $USER != "root"; then
	echo Please run this script as root, e.g. \'sudo sh $0\'.
	echo \(This is necessary in order to install executables.\)
	exit 1
fi

echo CREATING WORKING DIRECTORIES ----------------------------------
sudo -u $SUDO_USER mkdir build build/gmp build/mpfr build/binutils build/gcc build/newlib build/usb
if test $? -ne 0 ; then
	exit 1
fi

echo APPLYING PATCHES ----------------------------------------------
sudo -u $SUDO_USER patch -N -d ./src -p1 < patchfile
# No error checking on patch phase - user may be resuming a failed build.

echo CONFIGURING GMP LIBRARY ---------------------------------------
pushd build/gmp
sudo -u $SUDO_USER ../../src/gmp-*/configure ABI="32" --disable-shared
if test $? -ne 0 ; then
	exit 1
fi
echo MAKING GMP LIBRARY --------------------------------------------
sudo -u $SUDO_USER make CFLAGS="-fast" all
if test $? -ne 0 ; then
	exit 1
fi
echo INSTALLING GMP LIBRARY ----------------------------------------
make install
if test $? -ne 0 ; then
	exit 1
fi
popd

echo CONFIGURING MPFR LIBRARY --------------------------------------
pushd build/mpfr
sudo -u $SUDO_USER ../../src/mpfr-*/configure \
  LDFLAGS="-Wl,-search_paths_first" --disable-shared
if test $? -ne 0 ; then
	exit 1
fi
echo MAKING MPFR LIBRARY -------------------------------------------
sudo -u $SUDO_USER make CFLAGS="-fast" all
if test $? -ne 0 ; then
	exit 1
fi
echo INSTALLING MPFR LIBRARY ---------------------------------------
make install
if test $? -ne 0 ; then
	exit 1
fi
popd

echo CONFIGURING BINUTILS ------------------------------------------
pushd build/binutils
sudo -u $SUDO_USER ../../src/binutils-stable/configure \
  --prefix=/usr/local/stm32 --target=arm-none-eabi --disable-nls \
  --with-pkgversion="PYD 2008q3-39" \
  --with-bugurl="http://www.paintyourdragon.com/uc/osxstm32/"
if test $? -ne 0 ; then
	exit 1
fi
echo MAKING BINUTILS -----------------------------------------------
sudo -u $SUDO_USER make CFLAGS="-fast" all
if test $? -ne 0 ; then
	exit 1
fi
echo INSTALLING BINUTILS -------------------------------------------
make install
if test $? -ne 0 ; then
	exit 1
fi
popd

export PATH=$PATH:/usr/local/stm32/bin

echo CONFIGURING GCC -----------------------------------------------
pushd build/gcc
sudo -u $SUDO_USER ../../src/gcc-*/configure --prefix=/usr/local/stm32 \
  --target=arm-none-eabi --enable-interwork --enable-languages="c,c++" \
  --enable-multilib --disable-libgomp --disable-libmudflap --disable-libssp \
  --disable-libstdcxx-pch --disable-libunwind-exceptions --disable-nls \
  --disable-shared --disable-threads --with-float=soft --with-gnu-as \
  --with-gnu-ld --with-newlib --without-headers \
  --with-pkgversion="PYD 2008q3-39" \
  --with-bugurl="http://www.paintyourdragon.com/uc/osxstm32/"
if test $? -ne 0 ; then
	exit 1
fi
echo MAKING GCC \(COMPILER ONLY\) ------------------------------------
sudo -u $SUDO_USER make CFLAGS="-fast" CFLAGS_FOR_TARGET="-mcpu=cortex-m3 -mthumb" CXXFLAGS="-mcpu=cortex-m3 -mthumb" LIBCXXFLAGS="-mcpu=cortex-m3 -mthumb" all-gcc
if test $? -ne 0 ; then
	exit 1
fi
echo INSTALLING GCC \(COMPILER ONLY\) --------------------------------
make install-gcc
if test $? -ne 0 ; then
	exit 1
fi
popd

echo CONFIGURING NEWLIB --------------------------------------------
pushd src/newlib-stable
sudo -u $SUDO_USER autoconf
popd
pushd build/newlib
sudo -u $SUDO_USER ../../src/newlib-stable/configure \
  --prefix=/usr/local/stm32 --target=arm-none-eabi --enable-interwork \
  --enable-multilib --disable-newlib-atexit-dynamic-alloc --disable-nls \
  --disable-shared --with-float=soft --with-pkgversion="PYD 2008q3-39" \
  --with-bugurl="http://www.paintyourdragon.com/uc/osxstm32/"
if test $? -ne 0 ; then
	exit 1
fi
echo MAKING NEWLIB -------------------------------------------------
sudo -u $SUDO_USER make CFLAGS_FOR_TARGET="-ffunction-sections -fdata-sections -DPREFER_SIZE_OVER_SPEED -D__OPTIMIZE_SIZE__ -Os -fomit-frame-pointer -mcpu=cortex-m3 -mthumb -D__thumb2__ -D__BUFSIZ__=256" CCASFLAGS="-mcpu=cortex-m3 -mthumb -D__thumb2__"
if test $? -ne 0 ; then
	exit 1
fi
echo INSTALLING NEWLIB ---------------------------------------------
make install
if test $? -ne 0 ; then
	exit 1
fi
popd

echo MAKING GCC \(FULL\) ---------------------------------------------
pushd build/gcc
sudo -u $SUDO_USER make all
if test $? -ne 0 ; then
	exit 1
fi
echo INSTALLING GCC \(FULL\) -----------------------------------------
make install
if test $? -ne 0 ; then
	exit 1
fi
popd

echo CONFIGURING USB LIBRARY ---------------------------------------
pushd build/usb
sudo -u $SUDO_USER ../../src/libusb-*/configure
if test $? -ne 0 ; then
	exit 1
fi
echo MAKING USB LIBRARY --------------------------------------------
sudo -u $SUDO_USER make CFLAGS="-fast" all-am
if test $? -ne 0 ; then
	exit 1
fi
echo INSTALLING USB LIBRARY ----------------------------------------
make install-am
if test $? -ne 0 ; then
	exit 1
fi
popd

echo CLEANING UP ---------------------------------------------------
# Default installation is a bit sloppy with some duplicitous files.
# Clean that up and use a series of links in its place.
cd /usr/local/stm32/bin
rm -f arm-none-eabi-c++ arm-none-eabi-gcc-4.3.2
strip * &>/dev/null
ln -s arm-none-eabi-g++ arm-none-eabi-c++
ln -s arm-none-eabi-gcc arm-none-eabi-gcc-4.3.2
cd ../arm-none-eabi/bin
rm -f ar as c++ g++ gcc ld nm objcopy objdump ranlib strip
ln -s ../../bin/arm-none-eabi-ar ar
ln -s ../../bin/arm-none-eabi-as as
ln -s ../../bin/arm-none-eabi-g++ c++
ln -s ../../bin/arm-none-eabi-g++ g++
ln -s ../../bin/arm-none-eabi-gcc gcc
ln -s ../../bin/arm-none-eabi-ld ld
ln -s ../../bin/arm-none-eabi-nm nm
ln -s ../../bin/arm-none-eabi-objcopy objcopy
ln -s ../../bin/arm-none-eabi-objdump objdump
ln -s ../../bin/arm-none-eabi-ranlib ranlib
ln -s ../../bin/arm-none-eabi-strip strip
cd ../lib
rm -f libg.a
ln -s libc.a libg.a
cd armv6-m
rm -f libg.a
ln -s libc.a libg.a
cd ../thumb
rm -f libg.a crt0.o
ln -s libc.a libg.a
ln -s ../crt0.o crt0.o
cd ../thumb2
rm -f libg.a
ln -s libc.a libg.a
cd ../../../libexec/gcc/arm-none-eabi/4.3.2
strip c* install-tools/fixincl

echo SUCCESS! ------------------------------------------------------
echo If everything is compiled and installed to your satisfaction,
echo you can now delete the 'build' directory.  Before using the
echo compiler, remember to add this line to \~/.profile:
echo "    export PATH=\$PATH:/usr/local/stm32/bin"
echo The README file may contain additional usage notes.
exit 0
