/*
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*Example program that makes use of the adc and uart. Sends adc data every time there is incomming data from the usart1. Also sends a float number to the usart to test that floats are working.
  It initializes the system to 72MHz.
  Does a floating point operation for testing support for it.
  Sets the ports that are going to be used:
    LED to output
    Usart TX and TIM_ch1 as alt output
    Usart RX as FL input.
    ADC12 channel 1 as analog input.
  Sets the timer1 as output compare mode (this is a test function, a better one has to be developed)
  Initializes the Usart1 to 9600 baudrate
  Disables adc interrupt
  Turns off adc
  Initializes ADC
  Enables usart1 interrupt
  Enables globals interrupts and faults
  Then it loops doing nothing.
  The usart1 interrupt checks if there was a receive register full event:
     Then it read the new data
     Turns on the adc and waits for a conversion.
     The conversion gets formatted as characters and then sent to the usart1
*/
 
#include "main.h"
#include <stdbool.h>

void usart1_irq_handler(void) {
  uint16_t adc_data;
  char adc_char[60];
  for (int i=0;i<60;i++) {
    adc_char[i]=0;
  }
  //  static unsigned int counter=-10;
  static float counter_f=-4.34;
  int counter_int;
  int counter_decimal;
  unsigned char counter_sign;
  static bool first_time=true;
  int counter_temp;
  if (usart_receive_register_full_flag(USART1)) {
    if (!first_time) {
      unsigned char usart_data;
      usart_data=usart_data_read(USART1);
      
      //adquiring analog data
      adc_on(ADC1);
      while (!adc_eoc_flag(ADC1)){ }
      adc_data=adc_reg_data_read(ADC1);
      
      //Converting float number for printing
      counter_int=abs((int) counter_f);
      if (signbit(counter_f)!=0) {
	counter_sign='-';
      } else {
	counter_sign=' ';
      }
      counter_decimal=abs((int) (counter_f*10000-counter_int*10000));
      counter_f+=0.1;
      
      sprintf(adc_char, "\radc: 0x%03X %4u Float number test: %c%04d.%05d",adc_data, adc_data, counter_sign, counter_int, counter_decimal);

      usart_ndata_write(USART1,adc_char);
    } else {
      first_time=false;
    }
  }
}

void tim1_cc_irq_handler(void) {
  tim_cc_int_flag_clr(TIM1,TIM_CHANNEL1);
  static int toggle=0;
  if (toggle==0) {
    gpio_reset_pins(GPIOC,GPIO_Pin_12);
      //tim_force_update_event(TIM1);
      //tim_clock_enable(TIM1);
      //gpio_reset_pins(GPIOC,GPIO_Pin_2);
      //gpio_reset_pins(GPIOA,GPIO_Pin_8);
      toggle=1;
  } else {
    gpio_set_pins(GPIOC,GPIO_Pin_12);
    //gpio_set_pins(GPIOC,GPIO_Pin_2);
    //gpio_set_pins(GPIOA,GPIO_Pin_8);
    toggle=0;
  }
}

void usart_init(void) {
  usart_enable(USART1);
  usart_word_len_set(USART1,USART_8_N);
  usart_stop_bits_set(USART1,USART_1_STOP);
  usart_transmit_complete_int_disable(USART1);
  usart_transmit_enable(USART1);
  usart_baud_rate_set(USART1,9600,72000000);
  while (!usart_transmit_complete_flag(USART1)) {
  }
  usart_ndata_write(USART1,"\rTest!");
  usart_receive_enable(USART1);
  usart_receive_register_full_int_enable(USART1);
}

void adc_init(void) {
  adc_conf_t adc_config;
  while(adc_adon_flag(ADC1)) {}
  adc_on(ADC1);
  while(!adc_adon_flag(ADC1)) {}
  //wait tSTAB time
  adc_config=adc();
  adc_config.inj_ch_count=0;
  adc_set_conf(ADC1,adc_config);
  adc_regular_channel_sequence(ADC1,ADC_sequence1,ADC_channel1);
  adc_injected_channel_sequence(ADC1,ADC_sequence1,ADC_channel1);
  adc_inj_align_offset(ADC1,ADC_channel1,0);
  adc_sampling_time(ADC1,ADC_channel1,ADC_smp_1_5);
}

void system_init(void) {
  //CORE_disableGlobalFaultIRQ();
  // CORE_disableGlobalIRQ();
    gpio_set_mode(GPIOC,GPIOM_OUT50_PP,GPIO_Pin_12|GPIO_Pin_2);
    gpio_reset_pins(GPIOC,GPIO_Pin_12);
    rcc_oscOn(HSI);
    rcc_SYSCLKSourceSel(HSICLK);
    rcc_AHBPRESourceSel(SYSCLK);
    rcc_APB1PRESourceSel(HCLK);
    rcc_APB2PRESourceSel(HCLK);
    rcc_ADCPRESourceSel(PLCLK2DIV2);
    rcc_MCOSourceSel(NOCLK);
    rcc_oscOff(HSE);
    rcc_cssDisable();
    rcc_oscOff(PLL);
    rcc_oscBypDisable(HSE);
    rcc_PLLSourceSel(HSICLKDIV2);
    (void) rcc_PLLFreqScaleSel(2);
    rcc_USBPRESourceSel(PLLCLKDIV1_5);
    //clean RCC_CIR
    
    //72Mhz
    rcc_oscOn(HSE);
    while (rcc_is_oscReady(HSE));
    flash_prefetch_buf_enable();
    flash_latency(FLASH_latency_two);
    rcc_AHBPRESourceSel(SYSCLK);
    rcc_APB2PRESourceSel(HCLK);
    rcc_APB1PRESourceSel(HCLKDIV2);
    rcc_PLLSourceSel(HSICLKDIV2);
    (void) rcc_PLLFreqScaleSel(0);
    rcc_PLLSourceSel(HSECLK);
    (void) rcc_PLLFreqScaleSel(9);
    rcc_oscOn(PLL);
    while (rcc_is_oscReady(PLL));
    rcc_SYSCLKSourceSel(PLLCLK);
    while (rcc_sysclk_source_status()!=PLLCLK) {};
  (void) rcc_periClkEnable(IOPC);
  (void) rcc_periClkEnable(IOPA);
  (void) rcc_periClkEnable(RCC_TIM1); //56Mhz
  (void) rcc_periClkEnable(RCC_USART1); //56Mhz
  (void) rcc_periClkEnable(RCC_ADC1); //56/2Mhz
}

int main(void){
  system_init();
  volatile int A,C,D;
  D=10;
  C=10;
  int toggle=0;
  volatile float temp=0;
  temp=temp*12;
  for (int i=0;i<1000;i++) {
    temp+=0.01;
  }
  A=C+D;
  
  D=1;
  gpio_set_mode(GPIOC,GPIOM_OUT50_PP,GPIO_Pin_12|GPIO_Pin_2); // LED
  gpio_set_mode(GPIOA,GPIOM_OUT50_ALT_PP,GPIO_Pin_8|GPIO_Pin_9); //usart TX, TIM_ch1
  gpio_set_mode(GPIOA,GPIOM_IN_FL,GPIO_Pin_10); //usart RX
  gpio_set_mode(GPIOA,GPIOM_IN_AN,GPIO_Pin_1); //adc12 channel1
  gpio_set_pins(GPIOC,GPIO_Pin_12);
  
  tim_test_output_compare_mode(TIM1, TIM_CHANNEL1);
  usart_init(); 
  nvic_disable_irq(adc1_2_irq_IRQn);
  adc_off(ADC1);
  adc_init();
  nvic_enable_irq(usart1_irq_IRQn); 
  //nvic_enable_irq(tim1_cc_irq_IRQn);
  CORE_enableGlobalIRQ();
  CORE_enableGlobalFaultIRQ();
  
  //tim_test_forced_output_mode(TIM1,TIM_CHANNEL1);
  //    gpio_set_mode(GPIOA,GPIOM_OUT2_PP,GPIO_Pin_8);
  unsigned char usart_data='A';
  while (false) {
    while (usart_receive_register_full_flag(USART1)) {
      usart_data=usart_data_read(USART1);
      usart_data_write(USART1,usart_data);
      while (!usart_transmit_complete_flag(USART1)) {
      }
    }
  }
  while (false) {
    usart_data_write(USART1,usart_data);
    while (!usart_transmit_complete_flag(USART1)) {
    }
  }
  while (true) {
    if (toggle==0) {
      //usart_data_write(USART1,'A');
      //gpio_reset_pins(GPIOC,GPIO_Pin_12);
      //tim_force_update_event(TIM1);
      //tim_clock_enable(TIM1);
      //gpio_reset_pins(GPIOC,GPIO_Pin_2);
      //gpio_reset_pins(GPIOA,GPIO_Pin_8);
      toggle=1;
    } else {
      //usart_data_write(USART1,'B');
      //gpio_set_pins(GPIOC,GPIO_Pin_12);
      //gpio_set_pins(GPIOC,GPIO_Pin_2);
      //gpio_set_pins(GPIOA,GPIO_Pin_8);
      toggle=0;
    }
    for(int i=0;i<4000;i++) {
      for (int j=0;j<200;j++) {
	A=i;
	C=A+D;
      }
    }
  }
  return(SUCCESS);
}
