/*
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Example program that initializes the stm32 (clocks of all necessary systems. The core is set, I think, to the highest frequency. (72Mhz)), it uses float numbers to test the compilation and executing can make use of them. Then it sets the GPIOC Pin 12 as 50Mhz output, this output is connected to a led in the olimex-stm-h103 board. Then using two for loops to make waits we toogle the LED.*/
 
#include "main.h"
#include <stdbool.h>

void system_init(void) {
  //CORE_disableGlobalFaultIRQ();
  // CORE_disableGlobalIRQ();
    gpio_set_mode(GPIOC,GPIOM_OUT50_PP,GPIO_Pin_12|GPIO_Pin_2);
    gpio_reset_pins(GPIOC,GPIO_Pin_12);
    rcc_oscOn(HSI);
    rcc_SYSCLKSourceSel(HSICLK);
    rcc_AHBPRESourceSel(SYSCLK);
    rcc_APB1PRESourceSel(HCLK);
    rcc_APB2PRESourceSel(HCLK);
    rcc_ADCPRESourceSel(PLCLK2DIV2);
    rcc_MCOSourceSel(NOCLK);
    rcc_oscOff(HSE);
    rcc_cssDisable();
    rcc_oscOff(PLL);
    rcc_oscBypDisable(HSE);
    rcc_PLLSourceSel(HSICLKDIV2);
    (void) rcc_PLLFreqScaleSel(2);
    rcc_USBPRESourceSel(PLLCLKDIV1_5);
    //clean RCC_CIR
    
    //72Mhz
    rcc_oscOn(HSE);
    while (rcc_is_oscReady(HSE));
    flash_prefetch_buf_enable();
    flash_latency(FLASH_latency_two);
    rcc_AHBPRESourceSel(SYSCLK);
    rcc_APB2PRESourceSel(HCLK);
    rcc_APB1PRESourceSel(HCLKDIV2);
    rcc_PLLSourceSel(HSICLKDIV2);
    (void) rcc_PLLFreqScaleSel(0);
    rcc_PLLSourceSel(HSECLK);
    (void) rcc_PLLFreqScaleSel(9);
    rcc_oscOn(PLL);
    while (rcc_is_oscReady(PLL));
    rcc_SYSCLKSourceSel(PLLCLK);
    while (rcc_sysclk_source_status()!=PLLCLK) {};
  (void) rcc_periClkEnable(IOPC);
  (void) rcc_periClkEnable(IOPA);
  (void) rcc_periClkEnable(RCC_TIM1); //56Mhz
  (void) rcc_periClkEnable(RCC_USART1); //56Mhz
  (void) rcc_periClkEnable(RCC_ADC1); //56/2Mhz
}

int main(void){
  system_init();
  volatile int A,C,D;
  D=10;
  C=10;
  int toggle=0;
  volatile float temp=0;
  temp=temp*12;
  for (int i=0;i<1000;i++) {
    temp+=0.01;
  }
  A=C+D;
  D=1;
  gpio_set_mode(GPIOC,GPIOM_OUT50_PP,GPIO_Pin_12|GPIO_Pin_2); // LED
  gpio_set_pins(GPIOC,GPIO_Pin_12);
  
  while (true) {
    if (toggle==0) {
      gpio_reset_pins(GPIOC,GPIO_Pin_12);
      toggle=1;
    } else {
      gpio_set_pins(GPIOC,GPIO_Pin_12);
      toggle=0;
    }
    for(int i=0;i<400;i++) {
      for (int j=0;j<800;j++) {
	A=i;
	C=A+D;
      }
    }
  }
  return(SUCCESS);
}
