/*
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Example program that echos data from usart1. Also uses Tim1 to toogle LED and to generate PWM signal.
  Initializes clock to 72MHz
  Sets GPIOC Pin12 as output (LED)
  Sets GPIOA Pin 8 and 9 as ALT PP output (usart, tim)
  Sets GPIOA pin 10 as input (usart)
  Sets tim to output compare mode (check test function for timeout)
  Initizalizes usart 9600bps
  Enables usart1 and tim1 interrupts
  Enables interrupts and faults.
 */
 
#include "main.h"
#include <stdbool.h>

#define system_freq 72000000 //Hz
#define rcc_tim1_freq 72000000 //Hz
#define tim1_cycle_freq 1000 //tick cycle (Hz)
#define PI 3.1415926535 
volatile int freq=3;
int volt=100;

#define usart_buffer_size 100
unsigned char usart_buffer[usart_buffer_size];
int usart_buffer_read_pos=0;
int usart_buffer_write_pos=0;

void usart_push_to_buffer(unsigned char new_char) {
    usart_buffer[usart_buffer_write_pos]=new_char;
    usart_buffer_write_pos+=1;
    if (usart_buffer_write_pos> usart_buffer_size) {
      usart_buffer_write_pos=0;
    }
};

bool usart_pending_read(void) {
  if (usart_buffer_write_pos==usart_buffer_read_pos){
    return(false);
  } else { return(true);};
};

unsigned char  usart_pop_from_buffer(void) {
  unsigned char char_temp;
  if (usart_buffer_write_pos==usart_buffer_read_pos) {
    return('\0');
  }
  char_temp=usart_buffer[usart_buffer_read_pos];
  usart_buffer_read_pos+=1;
  if (usart_buffer_read_pos>usart_buffer_size){
    usart_buffer_read_pos=0;
  }
  return(char_temp);
}

void getline(unsigned char* line) {
  unsigned char ch='1';
  do {
    if (usart_pending_read()) {
      ch=usart_pop_from_buffer();
      *line++=ch;
      if (ch=='\r') {
	ch='\0';
	*line++=ch;
      }
    }
  } while (ch!='\0');
    
}

void usart1_irq_handler(void) {
  if (usart_receive_register_full_flag(USART1)) {
    unsigned char usart_data;
    usart_data=usart_data_read(USART1);
    usart_push_to_buffer(usart_data);
    //usart_data_write(USART1,usart_data);
    //while (!usart_transmit_complete_flag(USART1)) {
    //}
  }
}

void tim1_up_irq_handler(void) {
  gpio_reset_pins(GPIOC,GPIO_Pin_12);
  //tim_cc_int_flag_clr(TIM1,TIM_CHANNEL1);
  tim_uif_int_flag_clr(TIM1);
  static int toggle=0;
  static int toggle2=0;
  static int ticks=0;
  int max_ticks;
  max_ticks=tim1_cycle_freq/freq;
  volatile float duty_a, duty_b, duty_c, duty_an;
  float angle;
  
  angle=2*PI*ticks/max_ticks;
  duty_a=sinf(angle);
  duty_b=sinf(angle+2*PI/3);
  duty_c=sinf(angle+4*PI/3);
  
  //duty_a=0.5;
  //duty_b=0.5;
  //duty_c=0.5;
  
  if (duty_a < 0){
    TIM1->CCER&=~TIM_CC1E;
    TIM1->CCER|=TIM_CC1NE;
    duty_a=-duty_a;
  } else {
    TIM1->CCER|=TIM_CC1E;
    TIM1->CCER&=~TIM_CC1NE;
  }
  if (duty_b < 0){
    TIM1->CCER&=~TIM_CC2E;
    TIM1->CCER|=TIM_CC2NE;
    duty_b=-duty_b;
  } else {
    TIM1->CCER|=TIM_CC2E;
    TIM1->CCER&=~TIM_CC2NE;
  }
  if (duty_c < 0){
    TIM1->CCER&=~TIM_CC3E;
    TIM1->CCER|=TIM_CC3NE;
    duty_c=-duty_c;
  } else {
    TIM1->CCER|=TIM_CC3E;
    TIM1->CCER&=~TIM_CC3NE;
  }
  
  tim_ccr_write(TIM1,TIM_CHANNEL1, duty_a*rcc_tim1_freq/(2*tim1_cycle_freq)); //Capture compare register
  tim_ccr_write(TIM1,TIM_CHANNEL2, duty_b*rcc_tim1_freq/(2*tim1_cycle_freq)); //Capture compare register
  tim_ccr_write(TIM1,TIM_CHANNEL3, duty_c*rcc_tim1_freq/(2*tim1_cycle_freq)); //Capture compare register
  //tim_force_update_event(TIM1);
  
  if (toggle2==0) {
    toggle2=1;
  } else {
    toggle2=0;
  }

  
  ticks+=1;
  if (ticks>max_ticks) {
    if (toggle==0) {
      gpio_reset_pins(GPIOC,GPIO_Pin_2);
      toggle=1;
    } else {
      gpio_set_pins(GPIOC,GPIO_Pin_2);
      toggle=0;
    }
    ticks=0;
  }
  gpio_set_pins(GPIOC,GPIO_Pin_12);
    
}

void usart_init(void) {
  usart_enable(USART1);
  usart_word_len_set(USART1,USART_8_N);
  usart_stop_bits_set(USART1,USART_1_STOP);
  usart_transmit_complete_int_disable(USART1);
  usart_transmit_enable(USART1);
  usart_baud_rate_set(USART1,9600,72000000);
  while (!usart_transmit_complete_flag(USART1)) {
  }
  usart_ndata_write(USART1,"\rInit");
  usart_receive_enable(USART1);
  usart_receive_register_full_int_enable(USART1);
}

void system_init(void) {
  //CORE_disableGlobalFaultIRQ();
  // CORE_disableGlobalIRQ();
    gpio_set_mode(GPIOC,GPIOM_OUT50_PP,GPIO_Pin_12|GPIO_Pin_2);
    gpio_reset_pins(GPIOC,GPIO_Pin_12);
    rcc_oscOn(HSI);
    rcc_SYSCLKSourceSel(HSICLK);
    rcc_AHBPRESourceSel(SYSCLK);
    rcc_APB1PRESourceSel(HCLK);
    rcc_APB2PRESourceSel(HCLK);
    rcc_ADCPRESourceSel(PLCLK2DIV2);
    rcc_MCOSourceSel(NOCLK);
    rcc_oscOff(HSE);
    rcc_cssDisable();
    rcc_oscOff(PLL);
    rcc_oscBypDisable(HSE);
    rcc_PLLSourceSel(HSICLKDIV2);
    (void) rcc_PLLFreqScaleSel(2);
    rcc_USBPRESourceSel(PLLCLKDIV1_5);
    //clean RCC_CIR
    
    //72Mhz
    rcc_oscOn(HSE);
    while (rcc_is_oscReady(HSE));
    flash_prefetch_buf_enable();
    flash_latency(FLASH_latency_two);
    rcc_AHBPRESourceSel(SYSCLK);
    rcc_APB2PRESourceSel(HCLK);
    rcc_APB1PRESourceSel(HCLKDIV2);
    rcc_PLLSourceSel(HSICLKDIV2);
    (void) rcc_PLLFreqScaleSel(0);
    rcc_PLLSourceSel(HSECLK);
    (void) rcc_PLLFreqScaleSel(9);
    rcc_oscOn(PLL);
    while (rcc_is_oscReady(PLL));
    rcc_SYSCLKSourceSel(PLLCLK);
    while (rcc_sysclk_source_status()!=PLLCLK) {};

    //Turning on devices
    (void) rcc_periClkEnable(AFIOEN);
    (void) rcc_periClkEnable(IOPC);
    (void) rcc_periClkEnable(IOPA);
    (void) rcc_periClkEnable(IOPB);
    (void) rcc_periClkEnable(RCC_TIM1); //56Mhz
    (void) rcc_periClkEnable(RCC_USART1); //56Mhz
}

void gpio_afio_init(void) {
  //LED and pin C2 (for testing) as GPIO output
  gpio_set_mode(GPIOC,GPIOM_OUT50_PP,GPIO_Pin_12|GPIO_Pin_2);
  //USART output (alternate default)
  //gpio_set_mode(GPIOA,GPIOM_OUT50_ALT_PP,GPIO_Pin_8|GPIO_Pin_9);
  //USART input
  //gpio_set_mode(GPIOA,GPIOM_IN_FL,GPIO_Pin_10);

  //USART output in port B
  gpio_set_mode(GPIOB,GPIOM_OUT50_ALT_PP,GPIO_Pin_6);
  //USART input in port B
  gpio_set_mode(GPIOB,GPIOM_IN_FL,GPIO_Pin_7);

  //Turn on (off?) LED (test)
  gpio_set_pins(GPIOC,GPIO_Pin_12);
  //remap usart1 from port A to port B
  afio_USART1_enable_remap();
  
  //TIM1 outputs
  gpio_set_mode(GPIOB,GPIOM_OUT50_ALT_PP,GPIO_Pin_13|GPIO_Pin_14|GPIO_Pin_15);
  gpio_set_mode(GPIOA,GPIOM_OUT50_ALT_PP,GPIO_Pin_8|GPIO_Pin_9|GPIO_Pin_10);
}

void tim18_PWM1_init(tim_t* timer) {
  //Disables timer 
  tim_clock_disable(timer); //Disables counter
  tim_all_int_disable(timer); //Disable all interrupts
  tim_oc_main_output_disable(timer); //Disables compare outputs

  //sets tbu
  
  tim_tbu_conf(timer, TIM_CNT_M_CENTER_OUT_INT_UPDOWN, TIM_CNT_DIR_UP, TIM_CLK_M_INT,TIM_TRG_M_NONE, 0, 1, rcc_tim1_freq/(2*tim1_cycle_freq)); //TODO: We want to make rcc_tim1_freq/(2*tim1_cycle_freq) as big as possible (no more than 0xFFFF) so that we have the maximum resolution for the duty cycle. We could do this using the tim1 prescaler
  
   //Capture compare configuration. 
  
  tim_ccr_write(timer,TIM_CHANNEL1, 0.0*rcc_tim1_freq/(2*tim1_cycle_freq)); //Capture compare register
  tim_ccr_write(timer,TIM_CHANNEL2, 0.0*rcc_tim1_freq/(2*tim1_cycle_freq)); //Capture compare register
  tim_ccr_write(timer,TIM_CHANNEL3, 0.0*rcc_tim1_freq/(2*tim1_cycle_freq)); //Capture compare register

  //Configures output compare outputs to PWM1 (and other settings)
  tim_PWM1_oc_conf(timer, TIM_CHANNEL1, false);
  tim_PWM1_oc_conf(timer, TIM_CHANNEL2, false);
  tim_PWM1_oc_conf(timer, TIM_CHANNEL3, false);
  
  //Turns on the timer
  tim_update_int_enable(timer);
  tim_update_event_enable(timer);
  //tim_uif_int_flag_clr(timer);
  tim_oc_main_output_enable(timer);
  tim_force_update_event(timer);
  //tim_uif_int_flag_clr(timer);
  tim_clock_enable(timer);
  //tim_uif_int_flag_clr(timer);



  //tim_force_update_event(timer);
};

int main(void){
  system_init();
  gpio_afio_init();
  volatile int A,C,D;
  D=10;
  C=10;
  int toggle=0;
  volatile float temp=0;
  temp=temp*12;
  for (int i=0;i<1000;i++) {
    temp+=0.01;
  }
  A=C+D;
  D=1;
  
  //Comfigure timer for PWM
  tim18_PWM1_init(TIM1);
  usart_init();
  nvic_enable_irq(usart1_irq_IRQn); 
  nvic_enable_irq(tim1_up_irq_IRQn);
  CORE_enableGlobalIRQ();
  CORE_enableGlobalFaultIRQ();
  
  //tim_test_forced_output_mode(TIM1,TIM_CHANNEL1);
  //    gpio_set_mode(GPIOA,GPIOM_OUT2_PP,GPIO_Pin_8);
  unsigned char usart_data='A';
  while (false) {
    while (usart_receive_register_full_flag(USART1)) {
      usart_data=usart_data_read(USART1);
      usart_data_write(USART1,usart_data);
      while (!usart_transmit_complete_flag(USART1)) {
      }
    }
  }
  while (false) {
    usart_data_write(USART1,usart_data);
    while (!usart_transmit_complete_flag(USART1)) {
    }
  }
  unsigned char new_char[100];
  int value;
  bool up=true;

  while (true) {
    if (up) {
      freq++;
    } else {
      freq--;
    }
    if (freq==55) {
      up=false;
    }
    if (freq==0) {
      up=true;
    }
    /*getline(new_char);
    if (strcmp(new_char,"freq\r")==0) {
      usart_ndata_write(USART1, "\rEnter frequency\r\n");
      getline(new_char);
      sscanf(new_char, "%d", &value);
      freq=value;
      usart_ndata_write(USART1, new_char);
      }*/

    if (toggle==0) {
      //gpio_reset_pins(GPIOC,GPIO_Pin_12);
      //tim_force_update_event(TIM1);
      //tim_clock_enable(TIM1);
      //gpio_reset_pins(GPIOC,GPIO_Pin_2);
      //gpio_reset_pins(GPIOA,GPIO_Pin_8);
      toggle=1;
    } else {
      //usart_data_write(USART1,'B');
      //gpio_set_pins(GPIOC,GPIO_Pin_12);
      //gpio_set_pins(GPIOC,GPIO_Pin_2);
      //gpio_set_pins(GPIOA,GPIO_Pin_8);
      toggle=0;
    }
    for(int i=0;i<4000;i++) {
      value=freq+1+i;
      for (int j=0;j<200;j++) {
      usart_data_write(USART1,'A');
      value=freq*2+j+i;
      //freq++;
      }
    }
  }
  return(SUCCESS);
}
