/*                                                                              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>       
 *                                                                              
 * This program is free software; you can redistribute it and/or modify         
 * it under the terms of the GNU General Public License as published by         
 * the Free Software Foundation; either version 3 of the License, or            
 * (at your option) any later version.                                          
 *                                                                              
 * This program is distributed in the hope that it will be useful,              
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License            
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        
 */
#ifndef ADC_H
#define ADC_H

#include "mem_map.h"
#include <stdint.h>
#include "types.h"
#include "utils.h"

typedef struct {
  __RW uint32_t SR;
  __RW uint32_t CR1;
  __RW uint32_t CR2;
  __RW uint32_t SMPR1;
  __RW uint32_t SMPR2;
  __RW uint32_t JOFR1;
  __RW uint32_t JOFR2;
  __RW uint32_t JOFR3;
  __RW uint32_t JOFR4;
  __RW uint32_t HTR;
  __RW uint32_t LTR;
  __RW uint32_t SQR1;
  __RW uint32_t SQR2;
  __RW uint32_t SQR3;
  __RW uint32_t JSQR;
  __RW uint32_t JDR1;
  __RW uint32_t JDR2;
  __RW uint32_t JDR3;
  __RW uint32_t JDR4;
  __RW uint32_t DR;
} adc_t;

#define ADC1 ((adc_t *) ADC1_BASE)
#define ADC2 ((adc_t *) ADC2_BASE)
#define ADC3 ((adc_t *) ADC3_BASE)

typedef enum {
  ADC_sequence1=0,
  ADC_sequence2,
  ADC_sequence3,
  ADC_sequence4,
  ADC_sequence5,
  ADC_sequence6,
  ADC_sequence7,
  ADC_sequence8,
  ADC_sequence9,
  ADC_sequence10,
  ADC_sequence11,
  ADC_sequence12,
  ADC_sequence13,
  ADC_sequence14,
  ADC_sequence15,
  ADC_sequence16
} adc_sequence_t;

typedef enum {
  ADC_channel0=0,
  ADC_channel1,
  ADC_channel2,
  ADC_channel3,
  ADC_channel4,
  ADC_channel5,
  ADC_channel6,
  ADC_channel7,
  ADC_channel8,
  ADC_channel9,
  ADC_channel10,
  ADC_channel11,
  ADC_channel12,
  ADC_channel13,
  ADC_channel14,
  ADC_channel15,
  ADC_channel16,
  ADC_channel17
} adc_channel_t;

/*SR*/
#define ADC_JEOC bit2
#define ADC_EOC bit1

/*CR2*/
#define ADC_TSVREFE bit23
#define ADC_EXTTRIG bit20
#define ADC_EXTSEL_shift_places 17
#define ADC_EXTSEL_bit_mask 0x07
#define ADC_JEXTTRIG bit15
#define ADC_JEXTSEL_shift_places 12
#define ADC_JEXTSEL_bit_mask 0x07
#define ADC_ALIGN bit11
#define ADC_CAL bit2
#define ADC_CONT bit1
#define ADC_ADON bit0

typedef enum {
  ADC_align_right=false,
  ADC_align_left=true
} adc_align_t;

typedef enum {
  //for ADC1 and ADC2
  ADC_JEXTTRIG_adc12_tim1_trgo=0,
  ADC_JEXTTRIG_adc12_tim1_cc4,
  ADC_JEXTTRIG_adc12_tim2_trgo,
  ADC_JEXTTRIG_adc12_tim2_cc1,
  ADC_JEXTTRIG_adc12_tim3_cc4,
  ADC_JEXTTRIG_adc12_tim4_trgo,
  ADC_JEXTTRIG_adc12_exti15_tim8_cc4,
  ADC_JEXTTRIG_adc12_jswstart,
  //for ADC3
  ADC_JEXTTRIG_adc3_tim1_trgo=10,
  ADC_JEXTTRIG_adc3_tim1_cc4,
  ADC_JEXTTRIG_adc3_tim4_cc3,
  ADC_JEXTTRIG_adc3_tim8_cc2,
  ADC_JEXTTRIG_adc3_tim8_cc4,
  ADC_JEXTTRIG_adc3_tim5_trgo,
  ADC_JEXTTRIG_adc3_tim5_cc4,
  ADC_JEXTTRIG_adc3_jswstart
} adc_jextsel_t;

typedef enum {
  //for ADC1 and ADC2
  ADC_EXTTRIG_adc12_tim1_cc1=0,
  ADC_EXTTRIG_adc12_tim1_cc2,
  ADC_EXTTRIG_adc12_tim1_cc3,
  ADC_EXTTRIG_adc12_tim2_cc2,
  ADC_EXTTRIG_adc12_tim3_trgo,
  ADC_EXTTRIG_adc12_tim4_cc4,
  ADC_EXTTRIG_adc12_exti11_tim8_trgo,
  ADC_EXTTRIG_adc12_swstart,
  //for ADC3
  ADC_EXTTRIG_adc3_tim3_cc1=10,
  ADC_EXTTRIG_adc3_tim2_cc3,
  ADC_EXTTRIG_adc3_tim1_cc3,
  ADC_EXTTRIG_adc3_tim8_cc1,
  ADC_EXTTRIG_adc3_tim8_trgo,
  ADC_EXTTRIG_adc3_tim5_cc1,
  ADC_EXTTRIG_adc3_tim5_cc3,
  ADC_EXTTRIG_adc3_swstart
} adc_extsel_t;

/*CR1*/
#define ADC_DUALMOD_shift_places 16
#define ADC_DUALMOD_bit_mask 0x0F
#define ADC_DISCNUM_shift_places 13
#define ADC_DISCNUM_bit_mask 0x07
#define ADC_JDISCEN bit12
#define ADC_DISCEN bit11
#define ADC_JAUTO bit10
#define ADC_SCAN bit8
#define ADC_JEOCIE bit7
#define ADC_AWDIE bit6
#define ADC_EOCIE bit5

typedef enum {
  ADC_m_independent=0,
  ADC_m_reg_sim_inj_sim,
  ADC_m_reg_sim_alt_trig,
  ADC_m_inj_sim_fast_inter,
  ADC_m_inj_sim_slow_inter,
  ADC_m_inj_sim,
  ADC_m_reg_sim,
  ADC_m_fast_inter,
  ADC_m_slow_inter,
  ADC_m_alt_trig
} adc_dual_m_t;

/*SQRx*/
#define ADC_L_shift_places 20
#define ADC_L_bit_mask 0x0F
#define ADC_SQx_bit_mask 0x1F

/*JSQR*/
#define ADC_JL_shift_places 20
#define ADC_JL_bit_mask 0x03

/*JOFRx*/
#define ADC_JOFFSETx_shift_places 0
#define ADC_JOFFSETx_bit_mask 0x0FFF

/*SMPRx*/
#define ADC_SMPx_bit_mask 0x07

typedef enum {
  ADC_smp_1_5=0,
  ADC_smp_7_5,
  ADC_smp_13_5,
  ADC_smp_28_5,
  ADC_smp_41_5,
  ADC_smp_55_5,
  ADC_smp_71_5,
  ADC_smp_239_5,
} adc_smp_t;

typedef struct {
  uint32_t reg_ch_count; //regular channels used in scan mode. Use minimum one here
  uint32_t inj_ch_count; //injected channels used in scan mode. Use minimum one here
  bool auto_trigger; //Continuous mode
  bool scan;
  bool auto_injection;
  bool reg_split; //Discontinuous mode
  bool inj_split;
  adc_dual_m_t dual_mod;
  uint32_t reg_split_count; //Use minimum one here
  adc_align_t alignment;
  //uint32_t align_offset_ch1;
  //uint32_t align_offset_ch2;
  //uint32_t align_offset_ch3;
  //uint32_t align_offset_ch4;
  bool reg_ext_trigger;
  bool inj_ext_trigger;
  adc_extsel_t reg_exttrig_sel;
  adc_jextsel_t inj_exttrig_sel;
  bool temp_vref;
  bool reg_eoc_int;
  bool inj_eoc_int;
  bool watchdot_int;
} adc_conf_t;


void adc_set_conf(adc_t* adc, adc_conf_t conf);
adc_conf_t adc(void);
void adc_on(adc_t * adc); //not in conf
void adc_off(adc_t * adc); //not in conf
uint32_t adc_inj_data_read(adc_t * adc, adc_channel_t channel) ;
uint32_t adc_reg_data_read(adc_t * adc) ;
uint32_t adc_reg_data_adc2_read(adc_t * adc) ;
bool adc_adon_flag(adc_t * adc); //not in conf
bool adc_eoc_flag(adc_t * adc); //not in conf
bool adc_jeoc_flag(adc_t *adc); //not in conf
void adc_cal_start(adc_t* adc); //not in conf
bool adc_cal_status(adc_t* adc) ; //not in conf
int adc_inj_align_offset(adc_t *adc, adc_channel_t channel, uint32_t offset) ; //not in conf
int adc_sampling_time(adc_t *adc, adc_channel_t channel, adc_smp_t sampling_time) ; //not in conf
int adc_regular_channel_sequence(adc_t * adc, adc_sequence_t sequence, adc_channel_t channel); //not in conf
int adc_injected_channel_sequence(adc_t * adc, adc_sequence_t sequence, adc_channel_t channel); //not in conf
int adc_regular_channel_count(adc_t * adc, uint32_t count) ;
int adc_injected_channel_count(adc_t * adc, uint32_t count) ;
void adc_auto_trigger_enable(adc_t * adc) ;
void adc_auto_trigger_disable(adc_t * adc) ;
void adc_scan_enable(adc_t * adc) ;
void adc_scan_disable(adc_t * adc) ;
void adc_auto_injection_enable(adc_t * adc) ;
void adc_auto_injection_disable(adc_t * adc) ;
void adc_regular_split_mode_enable(adc_t *adc) ;
void adc_regular_split_mode_disable(adc_t *adc) ;
void adc_injected_split_mode_enable(adc_t *adc) ;
void adc_injected_split_mode_disable(adc_t *adc) ;
int adc_dual_mode(adc_t* adc, adc_dual_m_t mode) ;
int adc_reg_split_mode_num(adc_t * adc, uint32_t count) ;
void adc_align_set(adc_t *adc, adc_align_t alignment) ;
void adc_reg_ext_trigger_conversion_enable(adc_t *adc) ;
void adc_reg_ext_trigger_conversion_disable(adc_t *adc) ;
void adc_inj_ext_trigger_conversion_enable(adc_t *adc) ;
void adc_inj_ext_trigger_conversion_disable(adc_t *adc) ;
int adc_reg_ext_trigger_select(adc_t *adc, adc_extsel_t selection) ;
int adc_inj_ext_trigger_select(adc_t *adc, adc_jextsel_t selection) ;
void adc_temp_vref_enable(adc_t * adc) ;
void adc_temp_vref_disable(adc_t * adc) ;
void adc_reg_eoc_int_enable(adc_t * adc) ;
void adc_reg_eoc_int_disable(adc_t * adc) ;
void adc_inj_eoc_int_enable(adc_t * adc) ;
void adc_inj_eoc_int_disable(adc_t * adc) ;
void adc_watchdog_int_enable(adc_t * adc) ;
void adc_watchdog_int_disable(adc_t * adc) ;

#endif /*ADC_H*/

