/*                                                                                                                                       
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>                                                                
 *                                                                                                                                       
 * This program is free software; you can redistribute it and/or modify                                                                  
 * it under the terms of the GNU General Public License as published by                                                                  
 * the Free Software Foundation; either version 3 of the License, or                                                                     
 * (at your option) any later version.                                                                                                   
 *                                                                                                                                       
 * This program is distributed in the hope that it will be useful,                                                                       
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                                                                        
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                                         
 * GNU General Public License for more details.                                                                                          
 *                                                                                                                                       
 * You should have received a copy of the GNU General Public License                                                                     
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.                                                                 
 */

#ifndef GPIO_H
#define GPIO_H
#include "mem_map.h"
#include "types.h"
#include "utils.h"
#include <stdint.h>

typedef struct {
    __RW uint32_t CRL; //Control Register L
    __RW uint32_t CRH; //Control Register H
    __RW uint32_t IDR; //Input data register
    __RW uint32_t ODR; //Output data register
    __RW uint32_t BSRR; //Bit Set/Reset register
    __RW uint32_t BRR; //Bit Reset register
    __RW uint32_t LCKR; //Lock register
} gpio_t;

#define GPIOA ((gpio_t *) GPIOA_BASE)
#define GPIOB ((gpio_t *) GPIOB_BASE)
#define GPIOC ((gpio_t *) GPIOC_BASE)
#define GPIOD ((gpio_t *) GPIOD_BASE)
#define GPIOE ((gpio_t *) GPIOE_BASE)
#define GPIOF ((gpio_t *) GPIOF_BASE)
#define GPIOG ((gpio_t *) GPIOG_BASE)

typedef enum {
    GPIOM_IN_AN=0x0,
    GPIOM_IN_FL=(0x1<<2)|0x0,
    GPIOM_IN_UD=(0x2<<2)|0x0,
    GPIOM_IN_RE=(0x3<<2)|0x0,
    GPIOM_OUT10_PP=(0x0<<2)|0x1,
    GPIOM_OUT10_OD=(0x1<<2)|0x1,
    GPIOM_OUT10_ALT_PP=(0x2<<2)|0x1,
    GPIOM_OUT10_ALT_OD=(0x3<<2)|0x1,
    GPIOM_OUT2_PP=(0x0<<2)|0x2,
    GPIOM_OUT2_OD=(0x1<<2)|0x2,
    GPIOM_OUT2_ALT_PP=(0x2<<2)|0x2,
    GPIOM_OUT2_ALT_OD=(0x3<<2)|0x2,
    GPIOM_OUT50_PP=(0x0<<2)|0x3,
    GPIOM_OUT50_OD=(0x1<<2)|0x3,
    GPIOM_OUT50_ALT_PP=(0x2<<2)|0x3,
    GPIOM_OUT50_ALT_OD=(0x3<<2)|0x3,
    GPIOM_LAST=GPIOM_OUT50_ALT_OD
} gpio_mode_t;

#define GPIO_Pin_0 ((uint16_t)0x0001) 
#define GPIO_Pin_1 ((uint16_t)0x0002) 
#define GPIO_Pin_2 ((uint16_t)0x0004) 
#define GPIO_Pin_3 ((uint16_t)0x0008) 
#define GPIO_Pin_4 ((uint16_t)0x0010) 
#define GPIO_Pin_5 ((uint16_t)0x0020) 
#define GPIO_Pin_6 ((uint16_t)0x0040) 
#define GPIO_Pin_7 ((uint16_t)0x0080) 
#define GPIO_Pin_8 ((uint16_t)0x0100) 
#define GPIO_Pin_9 ((uint16_t)0x0200) 
#define GPIO_Pin_10 ((uint16_t)0x0400) 
#define GPIO_Pin_11 ((uint16_t)0x0800) 
#define GPIO_Pin_12 ((uint16_t)0x1000) 
#define GPIO_Pin_13 ((uint16_t)0x2000) 
#define GPIO_Pin_14 ((uint16_t)0x4000) 
#define GPIO_Pin_15 ((uint16_t)0x8000) 

/* LCKR*/
#define GPIO_LCKK bit16

static inline bool gpio_main_lock_read(gpio_t* gpio_port) {
  return(get_flagStatus(&(gpio_port->LCKR),GPIO_LCKK));
}

static inline bool gpio_lock_read(gpio_t* gpio_port, uint16_t bit) {
  return(get_flagStatus(&(gpio_port->LCKR),bit));
}

int gpio_set_mode(gpio_t* gpio_port,gpio_mode_t mode, uint16_t pins);

static inline void gpio_set_pins(gpio_t* gpio_port,uint16_t pins) {
    gpio_port->BSRR=((uint32_t) pins);
}


static inline void gpio_reset_pins(gpio_t* gpio_port,uint16_t pins) {
    gpio_port->BRR=((uint32_t) pins);
}

static inline void gpio_write(gpio_t* gpio_port,uint16_t data) {
    gpio_port->ODR=data;
}

static inline uint16_t gpio_read(gpio_t* gpio_port) {
    return(((uint16_t) gpio_port->IDR));
}

#endif /*GPIO_H*/
