

#ifndef NVIC_H
#define NVIC_H
#include "stm32f103.h"
#include "mem_map.h"
#include "types.h"
#include <stdint.h>


#define IRQ_CTRL_TYPE (NVIC_BASE + 0x00000004) /*read-only*/
#define SYSTICK_CTRL_STATUS (NVIC_BASE + 0x00000010)
#define SYSTICK_RELOAD (NVIC_BASE + 0x00000014)
#define SYSTICK_CURRENT (NVIC_BASE + 0x00000018)
#define SYSTICK_CAL (NVIC_BASE + 0x0000001C)
#define NVIC_IRQ_SET_EN_BASE (NVIC_BASE + 0x00000100)
#define NVIC_IRQ_CLR_EN_BASE (NVIC_BASE + 0x00000180)
#define NVIC_IRQ_SET_PEN_BASE (NVIC_BASE + 0x00000200)
#define NVIC_IRQ_CLR_PEN_BASE (NVIC_BASE + 0x00000280)
#define NVIC_IRQ_ACT_BIT_BASE (NVIC_BASE + 0x00000300)
#define NVIC_IRQ_PRI_BASE (NVIC_BASE + 0x00000400)
#define CPUID_BASE (NVIC_BASE + 0x00000D00)
#define IRQ_CTRL_STATE (NVIC_BASE + 0x00000D04)

static inline int nvic_irq_lines_num_read(void) {
  return((*((uint32_t *) IRQ_CTRL_TYPE))*32);
}

static inline int nvic_enable_irq(irq_t irq) {
  uint32_t reg_num;
  uint32_t shift_places;
  reg_num=irq/31;
  uint32_t * nvic_reg = ((uint32_t *) (NVIC_IRQ_SET_EN_BASE+(reg_num*sizeof(uint32_t))));
  if ((irq<0) || (irq>255)) { 
    return(ERROR);
  } else {
    shift_places=irq-reg_num*32;
    *nvic_reg|=(0x1 << shift_places);
  }
  return(SUCCESS);
}
static inline int nvic_disable_irq(irq_t irq) {
  uint32_t reg_num;
  uint32_t shift_places;
  reg_num=irq/31;
  uint32_t * nvic_reg = ((uint32_t *) (NVIC_IRQ_CLR_EN_BASE+(reg_num*sizeof(uint32_t))));
  if ((irq<0) || (irq>255)) { 
    return(ERROR);
  } else {
    shift_places=irq-reg_num*32;
    *nvic_reg|=(0x1 << shift_places);
  }
  return(SUCCESS);
}

static inline uint32_t cpuid_read(void) {
  return(*((uint32_t *) CPUID_BASE));
}



#endif /*NVIC_H*/
