/*              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *                
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *                                                  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.                                       
 *                      
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RCC_H
#define RCC_H
#include "types.h"
#include <stdint.h>
#include <stdbool.h>
#include "mem_map.h"

typedef struct {
    __RW uint32_t CR;
    __RW uint32_t CFGR;
    __RW uint32_t CIR;
    __RW uint32_t APB2RSTR;
    __RW uint32_t APB1RSTR;
    __RW uint32_t AHBENR;
    __RW uint32_t APB2ENR;
    __RW uint32_t APB1ENR;
    __RW uint32_t BDCR;
    __RW uint32_t CSR;
} rcc_t;

#define RCC ((rcc_t *) RCC_BASE)

typedef struct {
    //CR
    bool pllReady;
    bool hseReady;
    bool hsiReady;
    //CFGR
    //    sysclk_t sysclk; //reads from SWS
} rcc_status;

typedef struct {
    //CR
    bool pllOn;
    bool cssOn;
    bool hseOn;
    bool hsiOn;
    bool hseByp;
    uint16_t hsiCal;
    uint16_t hsiTrim;
    //CFGR
    //    mco_mode_t mco;
    //usbpre_t usbpre;
    //pllmul_t pllmul;
    //pll_hsepre_t pll_hsepre;
    //pllsrc_t pllsrc;
    //adcpre_t adcpre;
    //apb2pre_t apb2pre;
    //apb1pre_t apb1pre;
    //ahbpre_t ahbpre;
    //sysclk_t sysclk; //writes to SW
} rcc_conf;

/*osc enum*/
typedef enum {
    PLL, HSE, HSI, LSE, LSI
} osc_t;

/*clock signals*/
typedef enum {
  //RTC, MCO clock source:
  NOCLK,
  //SYS, MCO clock sources:
  HSICLK, 
  //SYS, MCO, PLL clock sources:
  HSECLK, 
  //PLL clock sources:
  HSECLKDIV2, HSICLKDIV2,
  //RTC clock source:
  HSECLKDIV128,
  //SYS, USB clock sources:
  PLLCLK, 
  //USB clock sources:
  PLLCLKDIV1_5, 
  //MCO clock sources:
  PLLCLKDIV2, 
  //ADC sources:
  PLCLK2DIV2, PLCLK2DIV4, PLCLK2DIV6, PLCLK2DIV8,
  //APB1 and APB2 sources:
  HCLK, HCLKDIV2, HCLKDIV4, HCLKDIV8, HCLKDIV16,
  //AHB, MCO sources:
  SYSCLK, 
  //AHB sources:
  SYSCLKDIV2, SYSCLKDIV4, SYSCLKDIV8, SYSCLKDIV16, SYSCLKDIV64,
  SYSCLKDIV128, SYSCLKDIV256, SYSCLKDIV512,
  //RTC clock sources:
  LSECLK, LSICLK
} clks_t;

typedef enum {
    //APB2
    APB2_FIRST=0, 
    AFIOEN=0, IOPA=2, IOPB, IOPC, IOPD, IOPE, IOPF, IOPG, RCC_ADC1, RCC_ADC2, RCC_TIM1=11,
    SPI1, RCC_TIM8, RCC_USART1, RCC_ADC3,
    APB2_LAST=RCC_ADC3,
    //APB1
    APB1_FIRST=100,
    TIM2=100, TIM3, TIM4, TIM5, TIM6, TIM7, WWDG=111, SPI2=114, SPI3,
    RCC_USART2=117, RCC_USART3, RCC_UART4, RCC_UART5, I2C1, I2C2, USB, CAN=125, BKP=127,
    PWR, DAC,
    APB1_LAST=DAC,
    //AHB
    AHB_FIRST=200,
    DMA1=200, DMA2, SRAM, FLITF=204, CRC=206, FSMC=208, SDIO=210,
    AHB_LAST=SDIO,
    //RTC
    RTC
} peri_t;

typedef enum {
    LPWRRST,
    WWDGRST,
    IWDGRST,
    SFTRST,
    PORRST,
    PINRST,
    RST_ERROR_REASON, //
} last_reset_t;

#define CASE( outval , unit , clksignal )               \
    case  clksignal :\
    outval = unit##_##clksignal ;\
    break

/*SYSCLK Status*/
#define SYSCLKSTATUS_shift_places 2
#define SYSCLKSTATUS_bit_mask 0x03
#define SYSCLKSTATUS_hsi 0x0
#define SYSCLKSTATUS_hse 0x1
#define SYSCLKSTATUS_pll 0x2


/*SYSCLK sources*/
#define SYSCLKSEL_shift_places 0
#define SYSCLKSEL_bits (bit0|bit1)
#define SYSCLKSEL_HSICLK 0x0
#define SYSCLKSEL_HSECLK 0x1
#define SYSCLKSEL_PLLCLK 0x2


/*APB2 prescaler*/
#define APB2PRE_shift_places 11
#define APB2PRE_bits (bit11|bit12|bit13)
#define APB2PRE_HCLK 0x0
#define APB2PRE_HCLKDIV2 0x4
#define APB2PRE_HCLKDIV4 0x5
#define APB2PRE_HCLKDIV8 0x6
#define APB2PRE_HCLKDIV16 0x7


/*APB1 prescaler*/
#define APB1PRE_shift_places 8
#define APB1PRE_bits (bit8|bit9|bit10)
#define APB1PRE_HCLK 0x0
#define APB1PRE_HCLKDIV2 0x4
#define APB1PRE_HCLKDIV4 0x5
#define APB1PRE_HCLKDIV8 0x6
#define APB1PRE_HCLKDIV16 0x7


/*AHB prescaler*/
#define AHBPRE_shift_places 4
#define AHBPRE_bits (bit4|bit5|bit6|bit7)
#define AHBPRE_SYSCLK 0x0
#define AHBPRE_SYSCLKDIV2 0x8
#define AHBPRE_SYSCLKDIV4 0x9
#define AHBPRE_SYSCLKDIV8 0xA
#define AHBPRE_SYSCLKDIV16 0xB
#define AHBPRE_SYSCLKDIV64 0xC
#define AHBPRE_SYSCLKDIV128 0xD
#define AHBPRE_SYSCLKDIV256 0xE
#define AHBPRE_SYSCLKDIV512 0xF

/*adc prescaler*/
#define ADCPRE_shift_places 14
#define ADCPRE_bits (bit14|bit15)
#define ADCPRE_PLCLK2DIV2 0x0
#define ADCPRE_PLCLK2DIV4 0x1
#define ADCPRE_PLCLK2DIV6 0x2
#define ADCPRE_PLCLK2DIV8 0x3

/*pll sources*/
#define PLLXTPRE_bits (bit17)
#define PLLSRC_bits (bit16)
#define PLLSEL_shift_places 16
#define PLLSEL_bits (PLLXTPRE_bits|PLLSRC_bits)
#define PLLSEL_HSICLKDIV2 0x0
#define PLLSEL_HSECLK 0x1
#define PLLSEL_HSECLKDIV2 0x3

/*pll multiplication factor*/
#define PLLMUL_shift_places 18
#define PLLMUL_bits (bit18|bit19|bit20|bit21)

/*usb prescales sources */
#define USBPRE_shift_places 22
#define USBPRE_bits (bit22)
#define USBPRE_PLLCLK 0x1
#define USBPRE_PLLCLKDIV1_5 0x0

/*MCO clock sources bits*/
#define MCOSEL_shift_places 24
#define MCOSEL_bits (bit24|bit25|bit26)
#define MCOSEL_NOCLK (0x0)
#define MCOSEL_SYSCLK (0x4)
#define MCOSEL_HSICLK (0x5)
#define MCOSEL_HSECLK (0x6)
#define MCOSEL_PLLCLKDIV2 (0x7)

/*RTC clock sources bits*/
#define RTCSEL_shift_places 8
#define RTCSEL_bits (bit8|bit9)
#define RTCSEL_NOCLK (0x0)
#define RTCSEL_LSECLK (0x1)
#define RTCSEL_LSICLK (0x2)
#define RTCSEL_HSECLKDIV128 (0x3)


/*osc ready bits*/
#define PLLRDY bit25
#define HSERDY bit17
#define HSIRDY bit1
#define LSERDY bit1 //RCC_BDCR
#define LSIRDY bit1 //RCC_CSR

/*osc on bits*/
#define PLLON bit24
#define HSEON bit16
#define HSION bit0
#define LSEON bit0 //RCC_BDCR
#define LSION bit0 //RCC_CSR

/*osc ready int clear bits*/
#define PLLRDYC bit20
#define HSERDYC bit19
#define HSIRDYC bit18
#define LSERDYC bit17
#define LSIRDYC bit16

/*osc ready int enable bits*/
#define PLLRDYIE bit12
#define HSERDYIE bit11
#define HSIRDYIE bit10
#define LSERDYIE bit9
#define LSIRDYIE bit8

/*osc ready int flag bits*/
#define PLLRDYF bit4
#define HSERDYF bit3
#define HSIRDYF bit2
#define LSERDYF bit1
#define LSIRDYF bit0

/*Clock security system bit*/
#define CSSON bit19

/*Clock security system int clear bit*/
#define CSSC bit23

/*Clock security system int flag bit*/
#define CSSF bit7

/*Osc bypass bits*/
#define HSEBYP bit18
#define LSEBYP bit2 //RCC_BDCR

/*HSI calibration and trim bits*/
#define HSICAL_bits (bit8|bit9|bit10|bit11|bit12|bit13|bit14|bit15)
#define HSICAL_shift_places 8
#define HSITRIM_bits (bit3|bit4|bit5|bit6|bit7)
#define HSITRIM_shift_places 3

/*peripheral reset bits*/
//APB2
/*
#define ADC3RST bit15
#define USART1RST bit14
#define TIM8RST bit13
#define SPI1RST bit12
#define TIM1RST bit11
#define ADC2RST bit10
#define ADC1RST bit9
#define IOPGRST bit8
#define IOPFRST bit7
#define IOPERST bit6
#define IOPDRST bit5
#define IOPCRST bit4
#define IOPBRST bit3
#define IOPARST bit2
#define AFIORST bit0
//APB1
#define DACRST bit29
#define PWRRST bit28
#define BKPRST bit27
#define CANRST bit25
#define USBRST bit23
#define I2C2RST bit22
#define I2C1RST bit21
#define UART5RST bit20
#define UART4RST bit19
#define USART3RST bit18
#define USART2RST bit17
#define SPI3RST bit15
#define SPI2RST bit14
#define WWDGRST bit11
#define TIM7RST bit5
#define TIM6RST bit4
#define TIM5RST bit3
#define TIM4RST bit2
#define TIM3RST bit1
#define TIM2RST bit0
*/

/*backup domain bits. RCC_BDCR*/
#define BDRST bit16
#define RTCEN bit15

/*last reset reason flags bits. RCC_CSR*/
#define LPWRRSTF bit31
#define WWDGRSTF bit30
#define IWDGRSTF bit29
#define SFTRSTF bit28
#define PORRSTF bit27
#define PINRSTF bit26
#define RESET_FLAG_BITS (LPWRRSTF|WWDGRSTF|IWDGRSTF|SFTRSTF|PORRSTF|PINRSTF)
#define RMVF bit24 //Remove reset reason

/*Legacy code, erase*/
#define AFOIEN bit0
#define IOPAEN bit2
#define IOPBEN bit3
#define IOPCEN bit4
#define IOPDEN bit5
#define IOPEEN bit6
#define IOPFEN bit7
#define IOPGEN bit8

/*oscilators functions*/
bool rcc_is_oscReady( osc_t osc);
int rcc_oscOn( osc_t osc);
int rcc_oscOff( osc_t osc);
void rcc_cssEnable(void);
void rcc_cssDisable(void);
void rcc_cssIntClear(void);
bool rcc_cssIntFlag(void);
int rcc_oscBypEnable( osc_t osc);
int rcc_oscBypDisable( osc_t osc);
int rcc_oscReadyIntClear(osc_t osc);
int rcc_oscReadyIntEnable(osc_t osc);
int rcc_oscReadyIntDisable(osc_t osc);
bool rcc_oscReadyIntFlag(osc_t osc);

/*pll Frequency scaling configuration function*/
int rcc_PLLFreqScaleSel( uint8_t factor);

/*HSI calibration*/
uint8_t rcc_HSICalRead(void);
uint8_t rcc_HSITrimRead(void);
int rcc_HSITrimWrite( uint8_t trimvalue);

/*clock source functions*/
int rcc_MCOSourceSel( clks_t clk);
int rcc_USBPRESourceSel( clks_t clk);
int rcc_PLLSourceSel( clks_t clk);
int rcc_ADCPRESourceSel( clks_t clk);
int rcc_APB2PRESourceSel( clks_t clk);
int rcc_APB1PRESourceSel( clks_t clk);
int rcc_AHBPRESourceSel( clks_t clk);
//uint8_t rcc_SYSCLKSourceRead(void);
clks_t rcc_sysclk_source_status(void);
int rcc_SYSCLKSourceSel( clks_t clk);
int rcc_RTCCLKSourceSel(clks_t clk);

/*Peripheral clock functions*/
int rcc_periReset(peri_t peri);
int rcc_periClkEnable(peri_t peri);
int rcc_periClkDisable(peri_t peri);

/*Backup domain functions*/
void rcc_backupDomReset(void);

/*Last reset reason flags*/
last_reset_t rcc_lastReset(void);
void rcc_clearLastReset(void);

#endif /*RCC_H*/
