/*              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *                
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *                                                  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.                                       
 *                      
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "stm32f103.h"

extern int main(void);

/* addresses defined in the linker script */
extern unsigned long _etext; /* end addr of .text section */
extern unsigned long _sidata; /* init values for .data section */
extern unsigned long _sdata; /* start addr of .data section */
extern unsigned long _edata; /* end addr of .data section */
extern unsigned long _sbss; /* start addr of .bss section */
extern unsigned long _ebss; /* end addr of .bss section */
extern void _estack; /* stack pointer init value */

void __reset_handler_stage1(void);
void __reset_handler_stage2(void);
void __null_handler(void);
void __default_exception_handler(void);

//Cortex-M3 exceptions handlers
void NonMaskableInt_exception_handler(void) __attribute__ ((weak, alias ("__default_exception_handler"))); 
void HardFault_exception_handler(void) __attribute__ ((weak, alias ("__default_exception_handler"))); 
void MemoryManagement_exception_handler(void) __attribute__ ((weak, alias ("__default_exception_handler"))); 
void BusFault_exception_handler(void) __attribute__ ((weak, alias ("__default_exception_handler"))); 
void UsageFault_exception_handler(void) __attribute__ ((weak, alias ("__default_exception_handler"))); 
void SVCall_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void DebugMonitor_irq_handler(void) __attribute__ ((weak, alias ("__null_handler"))); 
void PendSV_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void SysTick_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));

//STM32f103 irq handlers
void wwdg_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void pvd_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tamper_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void rtc_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void flash_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void rcc_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void exti0_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void exti1_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void exti2_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void exti3_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void exti4_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void dma1_channel1_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void dma1_channel2_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void dma1_channel3_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void dma1_channel4_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void dma1_channel5_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void dma1_channel6_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void dma1_channel7_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void adc1_2_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void usb_hp_can_tx_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void usb_lp_can1_rx0_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void can_rx1_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void can_sce_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void exti9_5_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim1_brk_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void tim1_up_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim1_trg_com_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void tim1_cc_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void tim2_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim3_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim4_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));  
void i2c1_ev_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void i2c1_er_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void i2c2_ev_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void i2c2_er_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void spi1_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void spi2_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void usart1_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void usart2_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void usart3_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void exti15_10_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void rtc_alarm_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void usb_wake_up_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim8_brk_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim8_up_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim8_trg_com_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim8_cc_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void adc3_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void fsmc_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void sdio_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim5_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void spi3_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void uart4_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void uart5_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim6_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void tim7_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void dma2_channel1_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void dma2_channel2_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void dma2_channel3_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));
void dma2_channel4_5_irq_handler(void) __attribute__ ((weak, alias ("__null_handler")));

/* interrupt vector */
__attribute__ ((section(".isr_vector")))
void (* const vector_table[])(void) = {
    // ARM Cortex-M3 exception handlers
    &_estack,              /* stack pointer init value*/
    __reset_handler_stage1,
    NonMaskableInt_exception_handler,
    HardFault_exception_handler,
    MemoryManagement_exception_handler,
    BusFault_exception_handler,
    UsageFault_exception_handler,
    0, 0, 0, 0,               /* reserved */
    SVCall_irq_handler,      
    DebugMonitor_irq_handler,
    0,                        /* reserved */
    PendSV_irq_handler,      
    SysTick_irq_handler,     

    // STM32 specific interrupt handlers
    wwdg_irq_handler,
    pvd_irq_handler,
    tamper_irq_handler,
    rtc_irq_handler,
    flash_irq_handler,
    rcc_irq_handler,
    exti0_irq_handler,
    exti1_irq_handler,
    exti2_irq_handler,
    exti3_irq_handler,
    exti4_irq_handler,
    dma1_channel1_irq_handler,
    dma1_channel2_irq_handler,
    dma1_channel3_irq_handler,
    dma1_channel4_irq_handler,
    dma1_channel5_irq_handler,
    dma1_channel6_irq_handler,
    dma1_channel7_irq_handler,
    adc1_2_irq_handler,
    usb_hp_can_tx_irq_handler,
    usb_lp_can1_rx0_irq_handler,
    can_rx1_irq_handler,
    can_sce_irq_handler,
    exti9_5_irq_handler,
    tim1_brk_irq_handler,
    tim1_up_irq_handler,
    tim1_trg_com_irq_handler,
    tim1_cc_irq_handler,
    tim2_irq_handler,
    tim3_irq_handler,
    tim4_irq_handler,
    i2c1_ev_irq_handler,
    i2c1_er_irq_handler,
    i2c2_ev_irq_handler,
    i2c2_er_irq_handler,
    spi1_irq_handler,
    spi2_irq_handler,
    usart1_irq_handler,
    usart2_irq_handler,
    usart3_irq_handler,
    exti15_10_irq_handler,
    rtc_alarm_irq_handler,
    usb_wake_up_irq_handler,
    tim8_brk_irq_handler,
    tim8_up_irq_handler,
    tim8_trg_com_irq_handler,
    tim8_cc_irq_handler,
    adc3_irq_handler,
    fsmc_irq_handler,
    sdio_irq_handler,
    tim5_irq_handler,
    spi3_irq_handler,
    uart4_irq_handler,
    uart5_irq_handler,
    tim6_irq_handler,
    tim7_irq_handler,
    dma2_channel1_irq_handler,
    dma2_channel2_irq_handler,
    dma2_channel3_irq_handler,
    dma2_channel4_5_irq_handler,
};

/* Get's called directly after mcu reset */
void __reset_handler_stage1(void){
    /* set stack align */
    //    SCB->CCR = SCB->CCR | SCB_CCR_STKALIGN;
    __reset_handler_stage2();
}

void __reset_handler_stage2(void){
    unsigned long *pul_src, *pul_dest;
    /* Copy the data segment initializers from flash to SRAM */
    pul_src = &_sidata;
    for(pul_dest = &_sdata; pul_dest < &_edata; ){
        *(pul_dest++) = *(pul_src++);
    }
    /* Zero fill the bss segment.  */
    for(pul_dest = &_sbss; pul_dest < &_ebss; ){
        *(pul_dest++) = 0;
    }
    /* Call the application's entry point.*/
    main();
}

void __null_handler(void){
    while(1);
}

void __default_exception_handler(void){
    while(1);
}
