/*              
 * Copyright (C) 2009 by Federico Ruiz-Ugalde <memeruiz at gmail dot com>
 *                
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *                                                  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.                                       
 *                      
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TIM_H
#define TIM_H

/*
Things to access:
- Counter register RW (TIMx_CNT)
- Prescaler Register RW (TIMx_PSC)
- Auto-Reload Register RW (TIMx_ARR)
- Repetition Counter Register (TIMx_RCR)
- Selection of mode (UP, Down, UP/Down, one-pulse-mode)
- Clock selection configuration (internal clock, external clock mode1, external clock mode2, internal trigger inputs)

- Capture/compare channels acceess RW (TIMx_)
- Capture/compare mode selection (input capture, PWM input, Forced output, output compare, PWM)
- PWM compare mode, mode selection (PWM edge-aligned, PWM center-aligned)
- Complementary output and dead-time configuration
- Break function 
- Encoder interface mode?
- Timer input XOR function?
- External trigger synchronization(reset mode, gated mode, trigger mode, external clock mode2+trigger mode)
-Timer synchronization


- Selection of the mode: input capture, output compare, PWM (edge,center aligned), complementary PWM with dead-time, one-pulse output)
- Syncronization

--------------------------

TIMx_CR1 Control Register 1
TIMx_CR2 Control Register 2
TIMx_SMCR Slave mode Control register
TIMx_DIER DMA/Interrupt enable register
TIMx_SR Status register
TIMx_EGR Event generation register
TIMx_CCMR1 Capture/compare mode register 1
TIMx_CCMR2 Capture/compare mode register 2
TIMx_CCER Capture/compare enable register
TIMx_CNT Counter
TIMx_PSC Prescaler
TIMx_ARR Auto-reload register
TIMx_RCR Repetition counter register
TIMx_CCR1 Capture/compare register 1
TIMx_CCR2 capture/compare register 2
TIMx_CCR3 capture/compare register 3
TIMx_CCR4 capture/compare register 4
TIMx_BDTR break and dead-time register
TIMx_DCR DMA control register
TIMx_DMAR DMA address for full transfer


 UDIS in TIMx_CR1 controls whether update events are generated or not at all. (ARR, PSC, CCRx) (useful for disabling update while the software is writing new data to ARR, PSC or CCRx)
 URS in TIMx_CR1 selects which one is the update event source. 0)counter over/underflow, setting UG, update gen through slave mode controller 1) counter over/underflow
 UIF in TIMx_SR indicates that an update ocurred.

               Time base unit( Counter, prescaler, auto reload register, repetition counter register) (TIMx_CNT, TIMx_PSC, TIMx_ARR, TIMx_RCR)

- Read and write (even while running)
- ARPE (TIMx_CR1) controls whether TIMx_ARR register is buffered or not
- Counter is clocked when CEN in TIMx_CR1 is set

- An update event happens when the counting is repeaded TIMx_RCR times.
- Setting UG (TIMx_EGR) generates an update event, even if URS in TIMx_CR1 is 1. (but no interrupt or DMA generation)

Upcounting mode:
Downcounting mode:
UP/Down:
   DIR in TIMx_CR1 can not be written. Can be read to find out the current direction of counting.

              Clock selection (internal clock CLK_INT, external clock mode1:external input pin, external clock mode2: external trigger pin ETR, internal trigger inputs ITRx)

internal clock source CLK_INT:
  if slave mode controller is disabled (SMS=000), CEN, DIR (TIMx_CR1) and UG (TIMx_EGR) are control bits. When CEN=1 then prescaler is clocked from CLK_INT.
External clock mode1:
  Configure the following:
     Input pin: CCxS (TIMx_CCMR1) to select (TI1 or TI2) or (TI3 or TI4)
     Filter duration: ICxF[3:0] (TIMx_CCMR1), no filter ICxF=0000
     Rising/Falling edge: CCxP (TIMx_CCER)
  SMS=111 in (TIMx_SMCR).
  trigger input source: TS (TIMx_SMCR) 
  Enable counter: CEN
External clock mode2:
  Filter duration: ETF[3:0] (TIMx_SMCR)
  Prescaler: ETPS[1:0] TIMx_SMCR
  Rising/Falling edge: ETP TIMx_SMCR
  ECE=1 in TIMx_SMCR
  Enable counter: CEN

              Capture/compare channels

  - Read/write to capture/compare registers (TIMx_CCRx)

Input capture mode:
Configuration:
  Select the active input: CCxS in TIMx_CCMR(1 or 2)
  Input Filter duration: ICxF in TIMx_CCMR(1 or 2)
  Select the edge (polarity) : CCxP TIMx_CCER
  Input prescaler: ICxPSC in TIMx_CCMR(1 or 2)
  Enable capture: CCxE in TIMx_CCER
  Enable interrupt request: CCyIE (TIMx_DIER)
  Enable DMA request: CCyDE (TIMx_DIER)
When capture occurs:
  TIMx_CCR1 gets count on active transition
  CCyIF is set. CCyOF is set if two consecutive captures occured and CCyIF not unset in first one. CCXIF can be cleared with 0 or by reading TIMx_CCRx, CCxOF is cleared writing 0.
  Interrupt (depends on CCyIE)
  DMA request (depends on CCyDE)

IC interrupt and/or DMA requests can be forced with CCxG (TIMx_EGR)

PWM input mode:
Same as input capture mode but two ICy signals  are mapped to same TIy input, these 2 ICy signals active on opposite edges, one of the TIyFP signals is selected as trigger input and the slave mode controller is configured in reset mode.
Configuration:
  Select the active input: CCyS (y=1 or 3) in TIMx_CCMR(1 or 2)
  Select the active polarity for TIzFPy (z=1 or 3) (y=1 or 3) (used both for capture TIMx_CCR(1 or 3) and counter clear)
  Select the edge (polarity rising) : CCyP (y=1 or 3) TIMx_CCER
  Select the active input: CCyS (y=2 or 4) in TIMx_CCMR(1 or 2)
  Select the active polarity for TIzFPy (z=1 or 3) (y=2 or 4) (used for capture TIMx_CCR(2 or 4))
  Select the edge (polarity falling) : CCyP (y=2 or 4) TIMx_CCER
  Select the valid trigger input: TS (TIMx_SMCR) to TIzFPy (zy=11 or 33)
  slave mode controller in reset mode: SMS=100 (TIMx_SMCR)
  ? = Input Filter duration: ICxF in TIMx_CCMR(1 or 2)
  ? = Input prescaler: ICxPSC in TIMx_CCMR(1 or 2)
  Enable capture: CCyE (y=(1 and 2) or (3 and 4) in TIMx_CCER)
  Enable interrupt request: CCyIE (TIMx_DIER)
  Enable DMA request: CCyDE (TIMx_DIER)

Forced output mode:
OCyREF is always active high
OCy opposite value of CCyP (polarity bit) is active state
Comparison between TIMx_CCRy and counter still happens. Sets the flag and can generate Ints and DMAs.
  Select output mode: CCyS = 00 (TIMx_CCMRz (z=1 or 2))
  OCyM = 101 (active level) 100 (inactive level) (TIMx_CCMRz (z=1 or 2))

Output compare mode
When a match happens:
  the output pin is set to the value programmed in OCyM (TIMx_CCMRz) and CCxP (TIMx_CCER)
  the output pin can keep its level (OCyM=000), set active (OCyM=001), set inactive (OCyM=010) or toogle (OCyM=011) on match.
  sets a flag CCyIF (TIMx_SR)
  generates an INT (dep CCyIE TIMx_DIER) and/or DMA (CCyDE TIMx_DIER, CCDS TIMx_CR2)
  UEV has no effect on OCyREF and OCy output
  Can also work as an output single pulse
Conf:
  Select the counter clock (internal, external, prescaler)
  Write data to TIMx_ARR, TIMx_CCRx
  Select if INTs CCyIE
  Select output mode (example):
    OCyM=011 (toogles OCy when match)
    OCyPE=0 disable preload register
    CCyP=0 select active high polarity
    CCyE=1 enable output
  Enable counter CEN (TIMx_CR1)

PWM mode:
  Select PWM mode: 110 (PWM mode 1) 111 (PWM mode 2) in OCyM (TIMx_CCMRz) 
  Enable the preload register: OCyPE (TIMx_CCMRz)
  Enable the auto-reload preload register: (in upcounting or centered-aligned modes) ARPE (TIMx_CR1)
  Initialize all registers from the preload registers: UG (TIMx_EGR)
  Set OCy polarity in CCyP (TIMx_CCER)
  Enable OCy: (CCyE, CCyNE, MOE, OSSI, OSSR) (TIMx_CCER and TIMx_BDTR)
  Choose PWM edge or center-aligned modes: CMS (TIMx_CR1)
  Choose Counting direction: DIR (TIMx_CR1) (only for edge-aligned mode)
Notes:  In center-aligned mode you can choose DIR before starting counting. DIR and CMS must not be change at the same time by software. Before start counting is recommended to generate an update by software (UG in (TIMx_EGR)) .

            Dead-time insertion (TODO)

            Break function (TODO)

            Clearing OCxREF signal on an external event (TODO)

            6-step PWM generation (TODO)

	    One-pulse mode (TODO)

	    Encoder interface mode(TODO)

	    TIMER input XOR function(TODO)

	    Interfacing with Hall sensors (TODO)

	    TIMx and external trigger synchronization (TODO)

	    Timer synchronization (TODO)

	    Debug mode (TODO)
*/
#include "types.h"
#include "mem_map.h"
#include <stdint.h>
#include "utils.h"

//Tim registers
typedef struct {
  __RW uint32_t CR1;
  __RW uint32_t CR2;
  __RW uint32_t SMCR;
  __RW uint32_t DIER;
  __RW uint32_t SR;
  __RW uint32_t EGR;
  __RW uint32_t CCMR1;
  __RW uint32_t CCMR2;
  __RW uint32_t CCER;
  __RW uint32_t CNT;
  __RW uint32_t PSC;
  __RW uint32_t ARR;
  __RW uint32_t RCR;
  __RW uint32_t CCR1;
  __RW uint32_t CCR2;
  __RW uint32_t CCR3;
  __RW uint32_t CCR4;
  __RW uint32_t BDTR;
  __RW uint32_t DCR;
  __RW uint32_t DMAR;
} tim_t;

#define TIM1 ((tim_t *) TIM1_BASE)
#define TIM8 ((tim_t *) TIM8_BASE)

//interrupt related functions
void tim_all_int_disable(tim_t* tim);
void tim_update_int_enable(tim_t* tim);
void tim_update_int_disable(tim_t* tim);
void tim_COM_int_enable(tim_t* tim);
void tim_COM_int_disable(tim_t* tim);
void tim_trigger_int_enable(tim_t* tim);
void tim_trigger_int_disable(tim_t* tim);

//Event interrupt related functions
void tim_update_event_enable(tim_t* tim); //(useful for disabling update while the software is writing new data to ARR, PSC or CCRx)
void tim_update_event_disable(tim_t* tim);
//UDIS CR1

typedef enum {
  TIM_UPEVENT_OVER_UNDER_UG_SLAVE,
  TIM_UPEVENT_OVER_UNDER
} tim_upevent_src_t;

void tim_update_event_source_select(tim_t* tim, tim_upevent_src_t upevent_source);  //0)counter over/underflow, setting UG, update gen through slave mode controller 1) counter over/underflow
//URS CR1

bool tim_update_event_interrupt_flag(tim_t* tim, bool clear); //an update event ocurred, clear the flag at end?
//UIF SR

//interrupt/dma enables for channels are in input and output sections down already,
//some other interrupt enables must be added here. Check DIER

//Time base unit related functions
typedef enum {
  TIM_CNT,
  TIM_PSC,
  TIM_ARR,
  TIM_RCR
} tim_tbur_t;
static inline uint16_t tim_tbu_read(tim_t* tim, tim_tbur_t reg) { //read from a time base unit register CNT, PSC, ARR and RCR
  switch (reg) {
  case TIM_CNT:
    return(tim->CNT);
    break;
  case TIM_PSC:
    return(tim->PSC);
    break;
  case TIM_ARR:
    return(tim->ARR);
    break;
  case TIM_RCR:
    return(tim->RCR);
    break;
  }
}
//CNT, PSC, ARR, RCR

static inline void tim_tbu_write(tim_t* tim, tim_tbur_t reg, uint16_t data) { //write to a time base unit register
  switch (reg) {
  case TIM_CNT:
    tim->CNT=data;
    break;
  case TIM_PSC:
    tim->PSC=data;
    break;
  case TIM_ARR:
    tim->ARR=data;
    break;
  case TIM_RCR:
    tim->RCR=data;
    break;
  }
}
//CNT, PSC, ARR, RCR. Always write 0 to the 8 high bits

void tim_arr_buffered_enable(tim_t* tim); //whether ARR register is buffered or not
void tim_arr_buffered_disable(tim_t* tim);
//ARPE (CR1)

void tim_clock_enable(tim_t* tim); //clock enable, is recommended to generate an UG (force update event) before start counting.
void tim_clock_disable(tim_t* tim);
//CEN (CR1)

void tim_force_update_event(tim_t* tim); //generates an update event.
//UG (EGR)

typedef enum {
  TIM_CNT_M_EDGE=0, //counts in one direction only
  TIM_CNT_M_CENTER_OUT_INT_DOWN=1, //counts up/down interrupt generation during down count
  TIM_CNT_M_CENTER_OUT_INT_UP=2, //counts up/down interrupt generation during up count
  TIM_CNT_M_CENTER_OUT_INT_UPDOWN=3 //counts up/down interrupt generation during up/down count
} tim_count_mode_t;

typedef enum {
  TIM_CNT_DIR_UP=false,
  TIM_CNT_DIR_DOWN=true
} tim_count_dir_t;

void tim_count_mode_sel(tim_t* tim, tim_count_mode_t count_mode, tim_count_dir_t init_count_dir); //selects the counting mode and the initial counting direction. For center-aligned mode, it should set the DIR bit only if the counter is disabled (CEN=OFF)
//CMS, DIR (CR1)

void tim_one_pulse_mode_enable(tim_t* tim);
void tim_one_pulse_mode_disable(tim_t* tim);
//OPM (CR1)

typedef enum {
  TIM_CLK_M_INT=0, //Clock comes from internal system clock
  TIM_CLK_M_ENC1=1, //Clock comes from encoder
  TIM_CLK_M_ENC2=2,
  TIM_CLK_M_ENC3=3,
  TIM_CLK_M_EXT1=7, //trigger from external clock selected by with trigger source select
  TIM_CLK_M_EXT2   //Trigger comes from ETRF signal (which is connected to TIMx_ETR pin indirectly
} tim_clk_mode_t;

typedef enum {
  TIM_TRG_M_NONE=0, //No special trigger mode selected. 
  TIM_TRG_M_RESET=4, //A rising edge on the trigger input produces a coutner reset
  TIM_TRG_M_GATED=5, //Counter is enable when trigger input is high. Start/stop of the counter is controlled.
  TIM_TRG_M_TRIGGER=6 //Counting starts at rising edge. Only starting the counter is controlled
} tim_trg_mode_t; //This things only work when using trigger inputs (TRGI), that's why they are called trigger modes.

//Clock selection and trigger related functions
int tim_clock_trigger_mode_select(tim_t* tim, tim_clk_mode_t clock_mode, tim_trg_mode_t trigger_mode); 
//selects the clock mode (encoder mode1 , encoder mode2, encoder mode3, external clock mode1 (trigger inputs), external clock mode2, internal clock mode)
//selects between the trigger mode types (none, reset, gated and trigger)
//Not all combinations work
//ECE (check documentation warnings) , SMS (SMCR)

typedef enum {
  TIM_TRG_SRC_ITR0,
  TIM_TRG_SRC_ITR1,
  TIM_TRG_SRC_ITR2,
  TIM_TRG_SRC_ITR3,
  TIM_TRG_SRC_TI1F_ED,
  TIM_TRG_SRC_TI1FP1,
  TIM_TRG_SRC_TI2FP2,
  TIM_TRG_SRC_ETRF,
} tim_trg_src_t;

void tim_trigger_source_select(tim_t* tim, tim_trg_src_t trigger_src); //selects the trigger signal source (internal triggers(ITR0-3), TI1 edge detector (TI1F_ED), Filtered timer input(1,2) (TI(1,2)FP(1,2)), external trigger input (ETRF)
//Change this only if SMS=000
//TS (SMCR)

typedef enum {
  ETPS_DIV1,
  ETPS_DIV2,
  ETPS_DIV4,
  ETPS_DIV8
} tim_etps_t;

typedef struct {
  bool polarity;
  tim_etps_t prescaler;
  uint8_t filter;  //only 4 bits
} tim_etr_t;

void tim_external_trigger_input_conf(tim_t* tim, tim_etr_t etr); //Configures the external trigger input circuitry
//ETP, ETPS, ETF (SMCR) tim_etr_t(struct)

//Capture/compare channels related functions
typedef enum {
  TIM_CHANNEL1,
  TIM_CHANNEL2,
  TIM_CHANNEL3,
  TIM_CHANNEL4,
} tim_channel_t;

static inline void tim_ccr_write(tim_t* tim, tim_channel_t channel, uint16_t data) { //writes data to the CCRy
  switch (channel) {
  case TIM_CHANNEL1:
    tim->CCR1=data;
    break;
  case TIM_CHANNEL2:
    tim->CCR2=data;
    break;
  case TIM_CHANNEL3:
    tim->CCR3=data;
    break;
  case TIM_CHANNEL4:
    tim->CCR4=data;
    break;
  }
}

static inline uint32_t tim_ccr_read(tim_t* tim, tim_channel_t channel, uint16_t data) { //reads data from the CCRy
  switch (channel) {
  case TIM_CHANNEL1:
    return(tim->CCR1);
    break;
  case TIM_CHANNEL2:
    return(tim->CCR2);
    break;
  case TIM_CHANNEL3:
    return(tim->CCR3);
    break;
  case TIM_CHANNEL4:
    return(tim->CCR4);
    break;
  }
}
void tim_cc_channel_event_gen(tim_t* tim, tim_channel_t channel); //generates an event by software (possible INT and DMA) in the specific channel. In capture mode it generates a capture.
//CCyG (EGR)

//input capture related

typedef enum {
  ICPS_DIV1,
  ICPS_DIV2,
  ICPS_DIV4,
  ICPS_DIV8
} tim_icps_t; //input channel prescaler values

typedef enum {
  ICSEL_TI1FP1,
  ICSEL_TI1FP2,
  ICSEL_TI2FP1,
  ICSEL_TI2FP2,
  ICSEL_TI3FP3,
  ICSEL_TI3FP4,
  ICSEL_TI4FP3,
  ICSEL_TI4FP4,
  ICSEL_TRC
} tim_icsel_t; //input sources

typedef struct {
  tim_icps_t prescaler; //ICyPSC (CCMR)
  tim_icsel_t input_sel; //CCyS (CCMR)
  bool polarity; // CCyP (CCER)
  uint8_t filter; // ICyF (CCMR)
  bool enable_capture; // CCyE (CCER)
  bool enable_int; //CCyIE (DIER)
  bool enable_dma; //CCyDE (CIER)
} tim_ic_t;

int tim_ic_conf(tim_t* tim, tim_channel_t channel, tim_ic_t ic_conf); //Configures the capture input channel
//Not all input sources are possible. It depends on the channel. check CCyS.
//returns error if Trigger input is not internal (check TS in trigger unit) when selecting TRC as input_sel and/or filter value bigger than FFFF, also returns an error if we are not in input compare mode (CCMRx.CCyS)

//General functions for all output compare channels
void tim_oc_main_output_enable(tim_t* tim); // enables the timer outputs
void tim_oc_main_output_disable(tim_t* tim);
//MOE (BDTR)

bool tim_oc_main_output_disabled(tim_t* tim); // Returns if the outputs are disabled (by software or by ext break input
//MOE (BDTR)

void tim_oc_main_output_autoenable_enable(tim_t* tim); //automatically enables outputs at the next update event
void tim_oc_main_output_autoenable_disable(tim_t* tim);
//AOE (BDTR)

void tim_oc_break_set_polarity(tim_t* tim, bool polarity);
//BKP (BDTR)
void tim_oc_break_enable(tim_t* tim); //enable break inputs (BRK and BRK_ACTH)
void tim_oc_break_disable(tim_t* tim); 
//BKE (BDTR)

//OSSR and OSSI not clear yet.
void tim_oc_breaked_outputs_disable(tim_t* tim); //outputs are disable
void tim_oc_breaked_outputs_force_idle_state(tim_t* tim); //outputs are forced to the programmed idle state (see OIS (CR2))
//OSSI (BDTR)
void tim_oc_not_breaked_outputs_disable(tim_t* tim); //outputs are disable
void tim_oc_not_breaked_outputs_force_idle_state(tim_t* tim); //outputs are forced to the programmed idle state (see OIS (CR2))
//OSSR (BDTR)

//Functions for specific output compare channels
typedef enum {
  OC_OUTPUT_M_DEAD_TIME_ACTIVE,
  OC_OUTPUT_M_OC_ACTIVE,
  OC_OUTPUT_M_OC_INACTIVE,
  OC_OUTPUT_M_OCN_ACTIVE,
  OC_OUTPUT_M_OC_OCN_INACTIVE
} tim_oc_output_mode_t; //we must set here CCyE and CCyNE (CCER) . Table 56

typedef enum {
  OC_MODE_FROZEN_ON_MATCH=0,
  OC_MODE_ACTIVE_ON_MATCH=1,
  OC_MODE_INACTIVE_ON_MATCH=2,
  OC_MODE_TOOGLE_ON_MATCH=3,
  OC_MODE_FORCE_INACTIVE=4, //set OCyM (CCMR) and that's it
  OC_MODE_FORCE_ACTIVE=5, 
  OC_MODE_PWM1=6, //difference between PWM1 and PWM2 modes is related to the matching polarity. PWM1 is active as long as TIMx_CNT<TIMx_CCRy, inactive as long as TIMx_CNT>TIMx_CCRy. In PWM2 is the other way around.
  OC_MODE_PWM2=7
} tim_oc_mode_t; //OCyM (CCMR)

//structure to set output compare configuration
typedef struct {
  tim_oc_mode_t oc_mode;
  tim_oc_output_mode_t oc_output_mode;
  bool oc_polarity; //CCyP (CCER)
  bool ocn_polarity; //CCyNP
  bool oc_idle_output_state; //OIS (CR2)
  bool ocn_idle_output_state; //OISN . Not available for channel 4
  bool oc_ocref_clear_on_ETRF; //(OCyCE (CCMR1)) check section 12.3.13, must return an error under certain circunstances. ETR input stage must be configure in a special way. 
  bool oc_preload_ccr; // (OCyPE(CCMR1)) Preload register on CCRy
  bool oc_fast_enable; // (OCyFE(CCMR1)) don't know what it does, works only in PWM(1,2) modes
  bool enable_int; // DIER.CCyIE
  bool enable_dma; // DIER.CCyDE
} tim_oc_t;

/* CCMR1 CCMR2 */
/*CC channel direction and input pin*/
#define TIM_CC1S_shift_places 0 //CCMR1
#define TIM_CC1S_bits (bit0|bit1)
#define TIM_CC2S_shift_places 8 //CCMR1
#define TIM_CC2S_bits (bit8|bit9)
#define TIM_CC3S_shift_places 0 //CCMR1
#define TIM_CC3S_bits (bit0|bit1)
#define TIM_CC4S_shift_places 8 //CCMR1
#define TIM_CC4S_bits (bit8|bit9)
#define TIM_CCyS_OUTPUT 0x0 //Channel configured as output
#define TIM_CCyS_INPUT_TI1 0x1
#define TIM_CCyS_INPUT_TI2 0x2
#define TIM_CCyS_INPUT_TRC 0x3 //only works if TS points to an internal trigger
/*Output compare mode*/
#define TIM_OC1M_shift_places 4
#define TIM_OC1M_bits (bit4|bit5|bit6)
#define TIM_OC2M_shift_places 12
#define TIM_OC2M_bits (bit12|bit13|bit14)
#define TIM_OC3M_shift_places 4
#define TIM_OC3M_bits (bit4|bit5|bit6)
#define TIM_OC4M_shift_places 12
#define TIM_OC4M_bits (bit12|bit13|bit14)
#define TIM_OC1FE bit2
#define TIM_OC1CE bit7
#define TIM_OC1PE bit3
#define TIM_OC2FE bit10
#define TIM_OC2CE bit15
#define TIM_OC2PE bit11
#define TIM_OC3FE bit2
#define TIM_OC3CE bit7
#define TIM_OC3PE bit3
#define TIM_OC4FE bit10
#define TIM_OC4CE bit15
#define TIM_OC4PE bit11

/* SR */
#define TIM_UIF bit0
#define TIM_CC1IF bit1
#define TIM_CC2IF bit2
#define TIM_CC3IF bit3
#define TIM_CC4IF bit4

static inline void tim_uif_int_flag_clr(tim_t* tim) {
  tim->SR&=~TIM_UIF;
}

static inline void tim_cc_int_flag_clr(tim_t* tim, tim_channel_t channel) {
  switch (channel) {
  case TIM_CHANNEL1:
    tim->SR&=~TIM_CC1IF;
    break;
  case TIM_CHANNEL2:
    tim->SR&=~TIM_CC2IF;
    break;
  case TIM_CHANNEL3:
    tim->SR&=~TIM_CC3IF;
    break;
  case TIM_CHANNEL4:
    tim->SR&=~TIM_CC4IF;
    break;
  }
  
}


/* CR1 */
#define TIM_CMS_shift_places 5  /* center-aligned mode selection */
#define TIM_CMS_bits (bit5|bit6)
#define TIM_DIR bit4
#define TIM_OPM bit3
#define TIM_UDIS bit1 /*update disable*/
#define TIM_CEN bit0 /* TIM_CR1 counter enable */

/* CR2 */
#define TIM_OIS1 bit8
#define TIM_OIS1N bit9
#define TIM_OIS2 bit10
#define TIM_OIS2N bit11
#define TIM_OIS3 bit12
#define TIM_OIS3N bit13
#define TIM_OIS4 bit14



/* EGR */
#define TIM_UG bit0 /* TIM_EGR update event */

/* SMCR */
#define TIM_ECE bit14 /* external clock enable */
#define TIM_SMS_shift_places 0 /*slave mode selection */
#define TIM_SMS_bits (bit0|bit1|bit2)
#define TIM_SMS_DIS 0x0
#define TIM_SMS_ENC1 0x1
#define TIM_SMS_ENC2 0x2
#define TIM_SMS_ENC3 0x3
#define TIM_SMS_RESET 0x4
#define TIM_SMS_GATED 0x5
#define TIM_SMS_TRIGGER 0x6
#define TIM_SMS_EXT1 0x7

/*CCER*/
#define TIM_CC1E bit0
#define TIM_CC2E bit4
#define TIM_CC3E bit8
#define TIM_CC4E bit12
#define TIM_CC1NE bit2
#define TIM_CC2NE bit6
#define TIM_CC3NE bit10
#define TIM_CC1P bit1
#define TIM_CC2P bit5
#define TIM_CC3P bit9
#define TIM_CC4P bit13
#define TIM_CC1NP bit3
#define TIM_CC2NP bit7
#define TIM_CC3NP bit11

/* DIER*/
#define TIM_UIE bit0;
#define TIM_COMIE bit5;
#define TIM_TIE bit6;
#define TIM_CC1IE bit1
#define TIM_CC1DE bit9
#define TIM_CC2IE bit2
#define TIM_CC2DE bit10
#define TIM_CC3IE bit3
#define TIM_CC3DE bit11
#define TIM_CC4IE bit4
#define TIM_CC4DE bit12

/* BDTR */
#define TIM_MOE bit15

int tim_oc_conf(tim_t* tim, tim_channel_t channel, tim_oc_t oc_conf); //configures the output compare channel
//here we must set CCyS to output (00)

//Todo bits:
//  CR1.CKD clock division
//  CR2.TI1S TI1 selection
//  CR2.MMS  Master mode selection
//  CR2.CCDS capture/compare DMA selection
//  CR2.CCUS capture/compare control update selection
//  CR2.CCPC capture/compare preload control
//  SMCR.MSM master/slave mode
//  DIER.TDE trigger DMA request enable
//  DIER.COMDE COM DMA request enable
//  DIER.UDE update DMA request enable
//  DIER.BIE break interrupt enable
//  DIER.TIE trigger interrupt enable
//  DIER.COMIE COM interrupt enable
//  DIER.UIE update interrupt enable
//  SR all except UIF flag
//  EGR.BG break generation
//  EGR.TG trigger generation
//  EGR.COMG capture/compare control update generation
//  BDTR.LOCK lock configuration
//  BDTR.DTG dead-time generator setup
//  DCR all
//  DMAR all 

//Special high level functions
//PWM input mode
//PWM modes
void tim_test_forced_output_mode(tim_t* timer, tim_channel_t channel) ;
void tim_test_output_compare_mode(tim_t* timer, tim_channel_t channel);
void tim_test_PWM1_mode(tim_t* timer, tim_channel_t channel);
void tim_tbu_conf(tim_t* timer, tim_count_mode_t count_mode, tim_count_dir_t init_count_dir, tim_clk_mode_t clock_mode, tim_trg_mode_t trigger_mode, uint16_t init_counter_value, uint16_t clock_prescaler, uint16_t auto_reload_value);
void tim_PWM1_oc_conf(tim_t* timer, tim_channel_t channel, bool enable_interrupt);

#endif /*TIM_H*/
