/*
 * Open-BLDC - Open BruschLess DC Motor Controller
 * Copyright (C) 2009 by Piotr Esden-Tempski <piotr@esden.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//#include <stdio.h> //for sprintf
//#include <string.h> //for memcpy

#include <stm32/rcc.h>
#include <stm32/flash.h>
#include <stm32/misc.h>
#include <stm32/gpio.h>
#include <stm32/spi.h>
#include <stm32/dma.h>

#include "usb.h"
#include "spi.h"

#include <stm32/usb/lib.h>
#include "usb_conf.h"
#include "usb_desc.h"
#include "usb_endp.h"

uint8_t SPI1_Buffer_Tx[BufferSize] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20};
uint8_t TxIdx = 0;

//Delcred in usb_endp.c
#define VIRTUAL_COM_PORT_DATA_SIZE              64
extern uint8_t buffer_in[VIRTUAL_COM_PORT_DATA_SIZE];
extern uint32_t count_in;


void rcc_init(void){
    /* Initialize the microcontroller system. Initialize clocks. */
    SystemInit();

    /* TIM1, GPIOA, GPIOB and GPIOC clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC |
                           RCC_APB2Periph_AFIO, ENABLE);

    usb_rcc_init();
    spi_rcc_init(); //might need to initialize a clock for the spi
}

void nvic_init(void){
    /* Set the Vector Table base location at 0x08000000 */
    NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);

    usb_nvic_init();
    //spi_nvic_init();  // Interrupt controller
}

void gpio_init(void){
    GPIO_InitTypeDef gpio;

    /* GPIOC: LED pin as output push-pull */
    GPIO_WriteBit(GPIOC,GPIO_Pin_12,Bit_SET);
    gpio.GPIO_Pin =  GPIO_Pin_12;
    gpio.GPIO_Mode = GPIO_Mode_Out_PP;
    gpio.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOC, &gpio);

    usb_gpio_init();
    spi_gpio_init();
}

void sys_tick_init(void){
    /* Generate SysTick interrupt every ?us (with HCLK=72MHz)
     *
     * We are using the SysTick only to force commutations for now so we trigger
     * the SysTick only when we need to commutate.
     */
    if(SysTick_Config(72*2*750*2)){
        while(1);
    }

    /* Set SysTick handler */
    NVIC_SetPriority(SysTick_IRQn, 0x0);
}

void my_delay(unsigned long delay ){

    while(delay){
        delay--;
    }
}


uint8_t read_byte_addr(uint8_t address) {
    
    GPIO_WriteBit(GPIOA, GPIO_Pin_4,0); //NSS down
    // Wait for SPI1 Tx buffer empty 
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
    // Send SPI1 data 
    SPI_I2S_SendData(SPI1, address);
    //wait for SPI1 transmission to end
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
    my_delay(50);

    //Read from the sensor
    uint8_t data=0;
    SPI_I2S_SendData(SPI1, 0); //Write a 0 to generate clock?
    data=SPI_I2S_ReceiveData(SPI1);
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
    my_delay(20);
    
    GPIO_WriteBit(GPIOA, GPIO_Pin_4,1); //NSS up
    return(data);
}


uint8_t read_byte_slave() {
    //Read from the sensor
    uint8_t data=0;
    data=SPI_I2S_ReceiveData(SPI1);
    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
    return(data);
}




int main(void){

    rcc_init(); /* system clocks init */
    nvic_init(); /* interrupt vector unit init */
    usb_init();
    gpio_init();
    spi_init();

    //DMA_Cmd(DMA1_Channel4, ENABLE);

    //TxIdx=0;
    //int counter=0;
    //while(1)
    //{
    //    counter++;
    //    GPIO_WriteBit(GPIOA, GPIO_Pin_4,0); //NSS down
    //    /* Transfer procedure */
    //    while (TxIdx < BufferSize)
    //    {
    //       /* Wait for SPI1 Tx buffer empty */
    //        while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
    //        /* Send SPI1 data */
    //        SPI_I2S_SendData(SPI1, SPI1_Buffer_Tx[TxIdx++]);
    //        //SPI_I2S_SendData(SPI1, SPI1_Buffer_Tx[4]);
    //    }
    //    TxIdx=0;
    //    //wait for SPI1 transmission to end
    //    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
    //    GPIO_WriteBit(GPIOA, GPIO_Pin_4,1); //NSS up
    //    my_delay(200);
    //}
    //uint8_t data=0;
    //while(1){
    //    data=read_byte_addr(0xF7);
    //    data=read_byte_addr(0xF8);
    //    data=read_byte_addr(0x00);
    //    data=read_byte_addr(0xF9);
    //    data=read_byte_addr(0xFA);
    //    data=read_byte_addr(0x00);
    //    data=read_byte_addr(0xFB);
    //    data=read_byte_addr(0xFC);
    //    data=read_byte_addr(0x00);
    //    my_delay(10);
    //    data=read_byte_addr(0xC3);
    //    data=read_byte_addr(0xC4);
    //    data=read_byte_addr(0xC5);
    //    data=read_byte_addr(0x00);

    //    my_delay(1000);
    //}




    //uint8_t buffer[11];
    //int i=0;
    //for (i=0 ; i < 11 ; i++) {
    //    buffer[i]=0;
    //}

    //for (i=0 ; i < 10 ; i++) {
    //    buffer[i]=read_byte_slave();
    //}


    //struct _reent main_reent;
    uint8_t data=0;
    uint8_t buffer[32];

    while (1){
        int read_count=16;
        int j=0;
        for (j=0 ; j<read_count ; j++) {
            buffer[j]=read_byte_slave();
        }
        int i=0;
        for (j=0 ; j<read_count ; j++) {
            data=buffer[j];
            buffer_in[i++]="0123456789abcdef"[((data)>>4)&0x0F];
            buffer_in[i++]="0123456789abcdef"[(data)&0x0F];
            buffer_in[i++]=' ';
        }
        buffer_in[i++]=0x0A; //0x0A == LineFeed
        buffer_in[i++]=0x0D; //0x0A == CarriageReturn
        buffer_in[i]=0;
        count_in=i;

        UserToPMABufferCopy(buffer_in, ENDP1_TXADDR, count_in);
        SetEPTxCount(ENDP1, count_in);
        SetEPTxValid(ENDP1);
        data=0;
    }

}



