/*
 * Open-BLDC - Open BruschLess DC Motor Controller
 * Copyright (C) 2009 by Alexis Maldonado <maldonad@cs.tum.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stm32/rcc.h>
#include <stm32/misc.h>
#include <stm32/gpio.h>
#include <stm32/spi.h>
#include <stm32/dma.h>

#include "spi.h"
uint8_t SPI_Buffer_Rx[BufferSize];

void spi_rcc_init(void){

    /* Enable peripheral clocks --------------------------------------------------*/
    /* DMA1 clock enable */
    //RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    /* SPI1 clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_SPI1, ENABLE);

}

void spi_nvic_init(void){
    //NVIC_InitTypeDef nvic;

    //nvic.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
    //nvic.NVIC_IRQChannelPreemptionPriority = 0;
    //nvic.NVIC_IRQChannelSubPriority = 0;
    //nvic.NVIC_IRQChannelCmd = ENABLE;
    //NVIC_Init(&nvic);
}

void spi_gpio_init(void){


    GPIO_PinRemapConfig(GPIO_Remap_SPI1, 0);

    GPIO_InitTypeDef GPIO_InitStructure;

    
    if (S_SPI_MASTER) {
        // Configure SPI1 pins: SCK, MISO and MOSI 
        GPIO_InitStructure.GPIO_Pin   =  GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP; //Pins connected to alternate function, Push-pull
        GPIO_Init(GPIOA, &GPIO_InitStructure);
        // Configure SPI1 pin: NSS  -> As regular GPIO output (push-pull) pin
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP; //Pin configured as output - push/pull
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_Init(GPIOA, &GPIO_InitStructure);
    
    } else {
        // Configure SPI1 pins: NSS, SCK, MISO and MOSI -> Connected to the SPI peripherial
        GPIO_InitStructure.GPIO_Pin   =  GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP; //Pins connected to alternate function, Push-pull
        GPIO_Init(GPIOA, &GPIO_InitStructure);
    }


}

void spi_init(void){

    SPI_InitTypeDef  SPI_InitStructure;

    /* SPI1 configuration ------------------------------------------------------*/
    if (S_SPI_MASTER){
        // SPI Master
        SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
        SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
        SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
        SPI_InitStructure.SPI_CPOL = SPI_CPOL_High; //We need high-level idle for SCK  CPOL=1
        SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge; //CPHA=1
        SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
        SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_128;
        SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
        SPI_InitStructure.SPI_CRCPolynomial = 7;
    } else {
        // SPI Slave
        SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
        SPI_InitStructure.SPI_Mode = SPI_Mode_Slave;
        SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
        SPI_InitStructure.SPI_CPOL = SPI_CPOL_High; //We need high-level idle for SCK  CPOL=1
        SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge; //CPHA=1
        SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;
        //SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_128;
        SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
        SPI_InitStructure.SPI_CRCPolynomial = 7;
    }
    
    SPI_Init(SPI1, &SPI_InitStructure);

    /* Enable SPI1 */
    SPI_Cmd(SPI1, ENABLE);


}


