/*
 * Open-BLDC - Open BruschLess DC Motor Controller
 * Copyright (C) 2009 by Piotr Esden-Tempski <piotr@esden.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stm32/rcc.h>
#include <stm32/misc.h>
#include <stm32/gpio.h>
#include <stm32/usb/lib.h>

#include "usb_conf.h"
#include "usb_istr.h"
#include "usb_pwr.h"
#include "usb_desc.h"
#include "usb_endp.h"

#include "usb.h"

#define USB_DISC_PIN GPIO_Pin_11
#define USB_DISC_PORT GPIOC

void usb_rcc_init(void){
    /* Enable USB_DISCONNECT GPIO clock */
    //RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_DISCONNECT, ENABLE);

    /* Configure the USB clock source */
    RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5);

    /* Enable the USB peripherial clock */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);
}

void usb_nvic_init(void){
    NVIC_InitTypeDef nvic;

    nvic.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
    nvic.NVIC_IRQChannelPreemptionPriority = 0;
    nvic.NVIC_IRQChannelSubPriority = 0;
    nvic.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&nvic);
}

void usb_gpio_init(void){
    GPIO_InitTypeDef gpio;

    /* Configure USB pull-up pin */
    gpio.GPIO_Pin   = USB_DISC_PIN;
    gpio.GPIO_Speed = GPIO_Speed_50MHz;
    gpio.GPIO_Mode  = GPIO_Mode_Out_OD;
    GPIO_Init(USB_DISC_PORT, &gpio);
}

void usb_init(void){
    USB_Init();
}

void usb_lp_can1_rx0_irq_handler(void){
    usb_istr();
}

void usb_enter_low_power(void){
    /* Set the device state to suspend */
    bDeviceState = SUSPENDED;
}

void usb_leave_low_power(void){
    DEVICE_INFO *pInfo = &Device_Info;

    /* Set the device state to the correct state */
    if(pInfo->Current_Configuration != 0){
        bDeviceState = CONFIGURED;
    }else{
        bDeviceState = ATTACHED;
    }
}

void usb_cable_config(FunctionalState new_state){
    if(new_state != DISABLE){
        GPIO_ResetBits(USB_DISC_PORT, USB_DISC_PIN);
    }else{
        GPIO_SetBits(USB_DISC_PORT, USB_DISC_PIN);
    }
}

void usb_get_serial(void){
    uint32_t device_serial0, device_serial1, device_serial2;

    device_serial0 = *(__IO uint32_t*)(0x1FFFF7E8);
    device_serial1 = *(__IO uint32_t*)(0x1FFFF7EC);
    device_serial2 = *(__IO uint32_t*)(0x1FFFF7F0);

    if(device_serial0 != 0){
        usb_serial[2] = (uint8_t)((device_serial0 &
                                   (0x7F << 0)) >> 0);
        usb_serial[4] = (uint8_t)((device_serial0 &
                                   (0x7F << 8)) >> 8);
        usb_serial[6] = (uint8_t)((device_serial0 &
                                   (0x7F << 16)) >> 16);
        usb_serial[8] = (uint8_t)((device_serial0 &
                                   (0x7F << 24)) >> 24);

        usb_serial[10] = (uint8_t)((device_serial1 &
                                   (0x7F << 0)) >> 0);
        usb_serial[12] = (uint8_t)((device_serial1 &
                                   (0x7F << 8)) >> 8);
        usb_serial[14] = (uint8_t)((device_serial1 &
                                   (0x7F << 16)) >> 16);
        usb_serial[16] = (uint8_t)((device_serial1 &
                                   (0x7F << 24)) >> 24);

        usb_serial[18] = (uint8_t)((device_serial2 &
                                   (0x7F << 0)) >> 0);
        usb_serial[20] = (uint8_t)((device_serial2 &
                                   (0x7F << 8)) >> 8);
        usb_serial[22] = (uint8_t)((device_serial2 &
                                   (0x7F << 16)) >> 16);
        usb_serial[24] = (uint8_t)((device_serial2 &
                                   (0x7F << 24)) >> 24);
    }
}

void usb_mirror_data(void){
    if((count_out != 0) && (bDeviceState == CONFIGURED)){
        UserToPMABufferCopy(buffer_out, ENDP1_TXADDR, count_out);
        SetEPTxCount(ENDP1, count_out);
        SetEPTxValid(ENDP1);
        count_out = 0;
    }
}
